%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per �berg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per �berg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function outValue =  psGeometryInit(method,arg1,arg2,arg3,arg4,arg5,arg6)
        
% Methods:
%   init                - Initializes psInitStruct
%   initSavedSimulation - Do initialization required for useage
%                         with a saved simulation
%   initData            - Initializes psSimStruct with predefined variables
%   createWindow        - Creates the window-needed.
%   updateWindow        - Updates the window
%   generatePlot        - Generates a plot in the plotting-area.
%   setParameters       - Set simulationParameters for submodule to arg1
%   saveDataClbck       - A callback to save simulationdata to file.
%   doneButtonClbck     - A callback to exit psXXXInit.m
%   cancelButtonClbck   - Another callback to exit psXXXInit.m
%   loadDataClbck       - A callback to load simulationdata from file.
%   changeValClbck      - Do this everytime parameters change
%   changeOptClbck      - Do this everytime an optimbox is clicked
%   changeUnitClbck     - Do this every time a unit-popup is used
%   changeVariantClbck  - Performs actions needed when another
%                         variant of initial data is chosen
%   createFcns          - Creted appropriate functions
%   checkType           - Should return: 'localFile','createLocal','generic'
%   getInitStruct       - Returns the initStruct. (For debugging purposes)

% ===========================================================================
%  Global initialization
% ===========================================================================
  persistent psInitStruct  
  global psSimStruct  
  outValue = 0;
  
  if nargin <1,
    disp('Youd should not run this command from the command-prompt.')
    method = '';
    outValue = -1;
  end
    
  
  % Global initialization: Change these according to your own specifications
  psGroup       = 'Geometry'; % Example "Geometry" (i.e. not "psGeometry")
  psSubGroup    = 'fixParGeometry';
  psPackVersion = '1.0';

  % For plotting
  plotDependentPars = {};
  
  % Method type: Options are 
  %   'localFile'
  %   'createLocal'
  %   'generic'
  methodType = 'createLocal'; 
  createsLocal = 1;
  
  % Window measures: (Measures in pixels) 
  % Read the documentation to see what the figures mean
  %  * Some distances
  framePixels = 4;
  axesPixels = 50;
  dx = 4 ;
  dy1 = 30;
  dy2 = 8;
  dy3 = 30;
  dy4 = 30; % Could be set to zero if no variants are available in "initData"
  dy5 = 10;
  dy6 = 10;
  y1 = 18;
  y2 = 30;
  x3 = 20;
  %  * Some total distances
  framePartSizeX  = 656;
  graphPartSizeX  = 0;
  graphPartMaxSizeY  = 200;
  
  %  * Some direved measures
  %    Pixels left to divide between the three is
  %    framePartSizeX-2*framePixels-5*dx-x3
  x1 = 115;
  x2 = 107;
  x4 = 107;
  x5 = 107;
  x6 = 60;
  x7 = 80;  
  %  x8+x9 < framePartSizeX-2*framePixels-3*dx
  x8 = 150;
  x9 = 150;

  freeColor = [0 0 0];
  uneditableColor = [1 0 0];

% ===========================================================================
% Methods
% ===========================================================================

  % -------------------------------------------------------------------------
  % Method      : init
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- working directory
  %               arg2 <- isVisual
  %               arg3 <- varStruct
  % -------------------------------------------------------------------------
  % Description : Performs necessary initialization.
  % -------------------------------------------------------------------------
  if strcmp(method,'init')
    if nargin < 2
      outValue = -1; 
      disp('You should specify a working Directory.')
      return;
    end

    % Since there are no global variables in this case
    % psSimStruct.psXXX should be cleared.
    psSimStruct.(['ps' psGroup]) = [];
    psSimStruct.(['ps' psGroup]).varInfo = [];
    psSimStruct.(['ps' psGroup]).varInfo.initialized = 0;
    psSimStruct.(['ps' psGroup]).varInfo.localCreated = 0;
    
    % Clear the variables.

    psInitStruct = [];
    psInitStruct.varStruct = [];
    psInitStruct.uihandleStruct = [];
    psInitStruct.variantHandling = [];
    psInitStruct.variantHandling.selectedVariant = 1;
    psInitStruct.callbackHandling= [];
    psInitStruct.callbackHandling.recentChangedVars = [];
    psInitSturct.initSettings = [];
    psInitStruct.initSettings.originalSimStruct = [];
    psInitStruct.initSettings.originalSimStruct.varInfo = [];
    psInitStruct.initSettings.originalSimStruct.varInfo.initialized = 0;
    
    % Take care of the in-varaibles
    psInitStruct.initSettings.workingDir = arg1;
    % Create window or not
    if nargin > 2
      psInitStruct.initSettings.isVisual = arg2;
    else
      psInitStruct.initSettings.isVisual = 1;
    end        
    
    %   Find the psXXXDir in use
    tmpXXXLoc = which(['ps' psGroup 'Init']);
    %       Just a precaution, in case there is a lower/upper problem...
    tmpXXXName = getfield(dir(tmpXXXLoc),'name');
    tmpIdx = findstr(tmpXXXName, tmpXXXLoc);    
    %       ... and finally we have the XXXDir
    psInitStruct.initSettings.(['ps' psGroup 'Dir']) = tmpXXXLoc(1:(tmpIdx-2));

    % Check if varStruct specified
    if nargin > 3
      tmpVarStruct = arg3;
    else
      tmpVarStruct = [];
    end        

    psInitStruct.initSettings.originalSimStruct = tmpVarStruct;
    eval(['ps' psGroup 'Init(''initData'');']);
    
    if ~isempty(tmpVarStruct) && ...
          isfield(tmpVarStruct,'varInfo') && ...
          isfield(tmpVarStruct.varInfo,'initialized') && ...
          tmpVarStruct.varInfo.initialized && ...
          isfield(tmpVarStruct.varInfo,'origin') && ...
          strcmp(tmpVarStruct.varInfo.origin,psSubGroup) &&...
          isfield(tmpVarStruct.varInfo,'psPackVersion') && ...
          strcmp(tmpVarStruct.varInfo.psPackVersion,psPackVersion)      
      psInitStruct.varStruct = tmpVarStruct;
    end
    
    % Make sure that a new window is created, (if it should be)
    psInitStruct.uihandleStruct.mainFig=0;
    
    eval(['ps' psGroup 'Init(''createWindow'');']);
    outValue = psInitStruct.uihandleStruct.mainFig;
  end
  % -------------------------------------------------------------------------
  % End Method: init
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : initSavedSimulation
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- distDir , arg2 <- workingDir , 
  %               arg3 <- psSimStruct.psXXX
  % -------------------------------------------------------------------------
  % Description : Performes initializations necessary when loading
  %               a simulation. If nothing needs to be done, just
  %               return the psXXX structure. For some cases
  %               this is where the
  %               psSimStruct.psXXX.varInfo.specialInfo is used.
  % -------------------------------------------------------------------------
  if strcmp(method,'initSavedSimulation')
    % Check that the calling of the command is correctly performed.
    if nargin < 4
      outValue = -1;
      disp(['You should specify a distribution-directory, working dir'...
            ' and a ' psGroup ' structure.'])
      return;
    end

    % Get variables from arguments
    distDir = arg1;
    workingDir = arg2;
    psXXXPar = arg3;
    
    % Do any necessary operations here.

    % Finish upp
    outValue = psXXXPar;
  end
  % -------------------------------------------------------------------------
  % End Method: initSavedSimulation
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : initData
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Initializes psInitStruct with predefined values.
  % -------------------------------------------------------------------------
  if strcmp(method, 'initData') 
    psInitStruct.variantHandling.numVariants = 1;
    psInitStruct.variantHandling.variantNames = {'SAAB 2.3l'};
    psInitStruct.varStruct = [];
    switch psInitStruct.variantHandling.selectedVariant;
     case 1
      psInitStruct.varStruct.varInfo.varNames=...
          {'a','l','B','V_c',...
           'n_cyl','r_c','V_d','V_D',...
           'S','Ath_IV','Ath_EV','Cd_IV',...
           'Cd_EV','EngineName'};  
      psInitStruct.varStruct.varInfo.varTypes=...
          {'value','value','value','value',...
           'value','value','value','value',...
           'value','value','value','value',...
           'value','string'};  
      psInitStruct.varStruct.varInfo.units=...
          {{'m','mm'},{'m','mm'},{'m','mm'},{'m3','liter'},...
           {'-'},{'-'},{'m3','liter'},{'m3','liter'},...
           {'m','mm'},{'m2','cm2'},{'m2','cm2'},{'-'},...
           {'-'},{''}};  
      psInitStruct.varStruct.varInfo.unitConversionFactors=...
          {{1,1e-3},{1,1e-3},{1,1e-3},{1,1e-3},...
           {1},{1},{1,1e-3},{1,1e-3},...
           {1,1e-3},{1,1e-4},{1,1e-4},{1},...
           {1},{1}};  
      psInitStruct.varStruct.varInfo.displayedUnitIdx=...
          {2,2,2,2,...
           1,1,2,2,...
           2,2,2,1,...
           1,1};  
      psInitStruct.varStruct.varInfo.varDesc = ...
          {'Cranc radius',...
           'Connecting rod length',...
           'Bore','Clearance Volume',...
           'Number of cylinders',...
           'Compression ratio',...
           'Displaced volume (one cylinder)',...
           'Displaced Volume (whole engine)',...
           'Piston stroke',...
           'Inlet valve area',...
           'Exhaust valve area',...
           'Discharge coefficient inlet valve',...
           'Discharge coefficient exhaust valve',...
           'Engine name'};
      psInitStruct.varStruct.varInfo.depVarNames  ={'S','V_d','V_D','r_c'};% Ber�kningsordning
      psInitStruct.varStruct.varInfo.depVarFuncs  =...
          {inline('2*a','a'),...
           inline('pi*B^2*S/4','B','S'),...
           inline('V_d*n_cyl','V_d','n_cyl'),...
           inline('(V_c+V_d)/V_c','V_c','V_d')};
      psInitStruct.varStruct.varInfo.freeVarNames =...
          {'a','l','B','V_c',...
           'n_cyl','Ath_IV','Ath_EV','Cd_IV',...
           'Cd_EV','EngineName'};
      psInitStruct.varStruct.varInfo.optimVarNames =...
          {'V_c','Ath_IV','Ath_EV','Cd_IV','Cd_EV'};
      psInitStruct.varStruct.varInfo.optimLimits =...
          {{0,0},{0,0},{0,0},{0,0},{0,0}};
      psInitStruct.varStruct.varInfo.optimShownInfo =...
          {'rel','rel','rel','rel','rel'};% Relative or absolute (rel/abs)
      psInitStruct.varStruct.varInfo.optimCtrl=...
          {'off','off','off','off','off'};
      
      psInitStruct.varStruct.varInfo.origin = psSubGroup;
      psInitStruct.varStruct.varInfo.psPackVersion = psPackVersion;
      % Set up the free variables initial values.
      initValuesFreeVars = {0.0450 ,0.1530 , 0.0900, 6.8983e-005,4, ...
                          0.0016,0.0012,0.8,0.8 ,...
                          'SAAB 2.3l naturally aspirated'};    
      [temp1,b,c]=intersect(psInitStruct.varStruct.varInfo.optimVarNames,psInitStruct.varStruct.varInfo.freeVarNames);
      optimInitLimits(b)   = initValuesFreeVars(c);
      for ii = 1:length(optimInitLimits(b))
          optimInitLimits{ii} = {optimInitLimits{ii}*0.95,optimInitLimits{ii}*1.05};
      end
      otherwise
      psInitStruct.variantHandling.selectedVariant = 1;
    end
    
    % Set the variables in the variable-struct
    for ii = 1:length(psInitStruct.varStruct.varInfo.freeVarNames)
      varName = psInitStruct.varStruct.varInfo.freeVarNames{ii};
      psInitStruct.varStruct.(varName) = initValuesFreeVars{ii};
    end

    for ii = 1:length(psInitStruct.varStruct.varInfo.optimVarNames)
        psInitStruct.varStruct.varInfo.optimLimits{ii} = optimInitLimits{ii};
    end

    % Calculate and set the non-free-variables.
    psInitStruct.varStruct = psCalcDepVars(psInitStruct.varStruct);    
  end
  % -------------------------------------------------------------------------
  % End Method: initData
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : createWindow
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Creates and draws necessary objects for the window.
  % -------------------------------------------------------------------------
  if strcmp(method,'createWindow')
    if psInitStruct.initSettings.isVisual
      % If needed, create the window.
      if ~psInitStruct.uihandleStruct.mainFig
        mainFig = figure('Visible','off');
        psInitStruct.uihandleStruct.mainFig = mainFig;
      else
        mainFig = psInitStruct.uihandleStruct.mainFig;
      end

      % Clear the uihandlestruct
      psInitStruct.uihandleStruct.variableList = [];

      % Calculate some necessary variables
      windowSizeX = framePartSizeX+graphPartSizeX;  
      nrBoxes =  length(psInitStruct.varStruct.varInfo.varNames)+1;
      windowSizeY = 2*framePixels+dy1+nrBoxes*y1+(nrBoxes-1)*dy2+dy3...
          +y1+dy4+3*y2+2*dy5+dy6;

      % Make the window look as I wish      
      mainFigPos = get(mainFig,'Position');  
      mainFigPos = [mainFigPos(1) ...
                    mainFigPos(2)+mainFigPos(4)-windowSizeY ...
                    windowSizeX windowSizeY ];
      set(mainFig,'MenuBar','none','Resize','off','Name',...
                  ['ps' psGroup ' - Parameter Setup'],'NumberTitle','off', ...
                  'Position',mainFigPos,'SelectionHighlight','off',...
                  'Units','pixels','Visible','on',...
                  'closeRequestFcn',['ps' psGroup 'Init(''cancelButtonClbck'',' num2str(mainFig) ');']);

      figure(mainFig);
      clf;

      % Draw the big frame
      uicontrol(mainFig,'Style','frame','Units','pixels',...
                'Position',[framePixels...
                            framePixels...
                            framePartSizeX-2*framePixels...
                            windowSizeY-2*framePixels]);  
      % Draw the lines 
      uicontrol(mainFig,'Style','frame','Units','pixels',...
                'Position',...
                [framePixels,...
                 windowSizeY-framePixels-dy1-y1-dy2-1,...
                 framePartSizeX-2*framePixels,...
                 y1+3]);  


      % Draw the title
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',[framePixels+dx...
                            windowSizeY-framePixels-dy1...
                            framePartSizeX-2*framePixels-2*dx...
                            y1],...
                'String',[psGroup  ' - ' psSubGroup], ...
                'SelectionHighlight','off','TooltipString',...
                'Enter geomtry parameters');  

      % Draw the variable-name title-box
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','Center',...
                'Position',...
                [framePixels+dx...
                 windowSizeY-framePixels-dy1-y1-dy2...
                 x1...
                 y1],...
                'String','Variable Name','SelectionHighlight','off', ...
                'TooltipString','The variable Name'); 
      % Draw the value-title-box followed by ...
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+2*dx+x1,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x2,...
                 y1],...
                'String','Value',...
                'SelectionHighlight','off',...
                'TooltipString','The value of the variable');
      % ... the optim-title-box and ...       
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+3*dx+x1+x2,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x3,...
                 y1],...
                'String','O','SelectionHighlight','off', ...
                'TooltipString',...
                ['Check this box if you want to'...
                 ' let the optimizer routine change this variable.']); 
      % ... the lower-limit-title-box followed by ...
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+4*dx+x1+x2+x3,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x4,...
                 y1],...
                'String','LowLim',...
                'SelectionHighlight','off',...
                'TooltipString','Lower limit during optimization');
      % ... the upper-limit-title-box followed by ...
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+5*dx+x1+x2+x3+x4,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x5,...
                 y1],...
                'String','UpLim',...
                'SelectionHighlight','off',...
                'TooltipString','Upper limit during optimization');
      % ... then the limit-showing-title-box
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+6*dx+x1+x2+x3+x4+x5,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x6,...
                 y1],...
                'String','LimType','SelectionHighlight','off', ...
                'TooltipString',...
                ['Choose to show absolute or relative optim limits']);
      
      % ... then the unit-title-box
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+6*dx+x1+x2+x3+x4+x5+x6,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x7,...
                 y1],...
                'String','Unit','SelectionHighlight','off', ...
                'TooltipString',...
                ['The selected unit. (Only for displaying purposes)']);
      
      for ii = 1:(length(psInitStruct.varStruct.varInfo.varNames))
        % ------------------------------------------
        % Set some control variables
        % ------------------------------------------

        % Set the main tooltip string
        toolTipString = psInitStruct.varStruct.varInfo.varDesc{ii};
        % Get the name of the variable
        varName = psInitStruct.varStruct.varInfo.varNames{ii};
        % Get the variable type
        varType = psInitStruct.varStruct.varInfo.varTypes{ii};
        % Get the value of the variable
        varValue =  psInitStruct.varStruct.(varName);
        % Set the unit-string and the box-type
        tmpUnits = psInitStruct.varStruct.varInfo.units{ii};
        unIdx = psInitStruct.varStruct.varInfo.displayedUnitIdx{ii};
        unitStr = [];
        if length(tmpUnits) > 1
          for tmpIdx = 1:length(tmpUnits)
            unitStr{tmpIdx} = ['[' tmpUnits{tmpIdx} ']'];
          end
          unitBoxType = 'popupmenu';
        else          
          unitStr=['[' tmpUnits{unIdx} ']'];
          unitBoxType = 'text';
        end
        % Manipulate the value to a fitting string
        switch varType
         case 'table'
          if isnumeric(varValue) && varValue == 0
            varStr = [varName ' table NOT loaded...'];            
          else 
            varStr = [varName ' table loaded...'];
          end
         case 'string'
          if ~isstr(varValue) 
            varStr = [varName ' not Found...'];            
          else 
            varStr = varValue;
          end
         case 'value'
          tmpFactor = ...
              psInitStruct.varStruct.varInfo.unitConversionFactors{ii}{unIdx};
          varStr = num2str(varValue/tmpFactor);
        end
        % Set colour of text and style of valuebox. In case one
        % wants to change to a textbox in the future. Also change
        % the tooltip string if the variable is uneditable
        if find(strcmp(psInitStruct.varStruct.varInfo.freeVarNames,varName))
          valueStyleString = 'edit';
          theColour = freeColor;
        else
          valueStyleString = 'edit';
          theColour = uneditableColor;
          toolTipString = [toolTipString ' (Derived/Uneditable parameter)'];
        end
        % Get optimCtrlStr
        optimVarIdx = ...
            find(strcmp(psInitStruct.varStruct.varInfo.optimVarNames,varName));
        isOptimVar = ~isempty(optimVarIdx);
        if isOptimVar
          optimCtrlStr = ...
              psInitStruct.varStruct.varInfo.optimCtrl{optimVarIdx};
        
          upLim = psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2};
          lowLim = psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1};
          if strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'abs')
              upLimFinal = upLim/tmpFactor;
              lowLimFinal = lowLim/tmpFactor;
          else
            if varValue
              upLimFinal = upLim/varValue;
              lowLimFinal = lowLim/varValue;
            else
              upLimFinal = NaN;
              lowLimFinal = NaN;
            end
          end
          upLimStr  = num2str(upLimFinal);
          lowLimStr = num2str(lowLimFinal);
        else 
          optimCtrlStr = '';
        end

        % ------------------------------------------
        % Start drawing
        % ------------------------------------------
        
        % Draw the variable Name
        uicontrol(mainFig,'Style','text',...
                  'Units','pixels',...
                  'HorizontalAlignment','left',...
                  'Position',...
                  [framePixels+dx...
                   windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2)...
                   x1...
                   y1],...
                  'String',varName,'SelectionHighlight','off', ...
                  'TooltipString',toolTipString); 
        % Draw the value box and the unit OR only the variable box
        % Possible types are 'value','table' and 'string'
        switch varType
         case 'table'
          % Draw the value box only since strings or a tables
          % are not optimizable.
          psInitStruct.uihandleStruct.variableList.(varName).value=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','center',...
                        'ForegroundColor',theColour,'Position',...
                        [framePixels+2*dx+x1...
                       windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2)...
                         x2+x3+x7+2*dx...
                         y1],...
                        'Enable','off',...
                        'String',varStr,...
                        'Callback',['ps' psGroup 'Init(''changeValClbck'');'],...
                        'SelectionHighlight','off',...
                        'TooltipString','Uneditable table or struct loaded');
         case 'string'
          % Draw the value box only since strings or a tables
          % are not optimizable.
          psInitStruct.uihandleStruct.variableList.(varName).value=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','center',...
                        'ForegroundColor',theColour,'Position',...
                        [framePixels+2*dx+x1...
                       windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2)...
                         x2+x3+x7+2*dx...
                         y1],...
                        'String',varStr,...
                        'Callback',['ps' psGroup 'Init(''changeValClbck'');'],...
                        'SelectionHighlight','off',...
                        'TooltipString',toolTipString);
         case 'value'
          % Draw the value-box followed by ...
          psInitStruct.uihandleStruct.variableList.(varName).value=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','right',...
                        'ForegroundColor',theColour,'Position',...
                        [framePixels+2*dx+x1,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x2,...
                         y1],...
                        'String',varStr,...
                        'Callback',['ps' psGroup 'Init(''changeValClbck'');'],...
                        'SelectionHighlight','off',...
                        'TooltipString',toolTipString);
          % ... the optimbox and ... 
          if isOptimVar                        
          optimIsOn =strcmp(optimCtrlStr,'on');
          psInitStruct.uihandleStruct.variableList.(varName).opt = ...
              uicontrol(mainFig,'Style','checkbox',...
                        'Units','pixels',...
                        'HorizontalAlignment','left',...
                        'Position',...
                        [framePixels+3*dx+x1+x2,...
                         windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x3,...
                         y1],...
                        'String','','SelectionHighlight','off', ...
                        'Callback',['ps' psGroup 'Init(''changeOptClbck'');'],...
                        'TooltipString',toolTipString,...
                        'Value',optimIsOn); 
          % ... then the lower-limit-value-box
          psInitStruct.uihandleStruct.variableList.(varName).lowlim=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','right',...
                        'ForegroundColor',0.5*(1+theColour-optimIsOn*(1-theColour)),'Position',...
                        [framePixels+4*dx+x1+x2+x3,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x4,...
                         y1],...
                        'String',lowLimStr,...
                        'Callback',['ps' psGroup 'Init(''changeValClbck'');'],...
                        'SelectionHighlight','off',...
                        'TooltipString',toolTipString);
          % ... then the upper-limit-value-box
          psInitStruct.uihandleStruct.variableList.(varName).uplim=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','right',...
                        'ForegroundColor',0.5*(1+theColour-optimIsOn*(1-theColour)),'Position',...
                        [framePixels+5*dx+x1+x2+x3+x4,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x5,...
                         y1],...
                        'String',upLimStr,...
                        'Callback',['ps' psGroup 'Init(''changeValClbck'');'],...
                        'SelectionHighlight','off',...
                        'TooltipString',toolTipString);
          % ... then the limtype-box
          psInitStruct.uihandleStruct.variableList.(varName).limtype = ...
              uicontrol(mainFig,'Style','popupmenu',...
                        'Units','pixels','Value',1+strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'rel'),...
                        'HorizontalAlignment','center',...
                        'ForegroundColor',0.5*(1+theColour-optimIsOn*(1-theColour)),'Position',...
                        [framePixels+6*dx+x1+x2+x3+x4+x5,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x6,...
                         y1],...
                        'String',{'abs','rel'},'SelectionHighlight','off', ...
                        'Callback',['ps' psGroup 'Init(''changeValClbck'');'],...
                        'TooltipString',toolTipString); 
          end
          % ... then the unit-box
          psInitStruct.uihandleStruct.variableList.(varName).unit = ...
              uicontrol(mainFig,'Style',unitBoxType,...
                        'Units','pixels','Value',unIdx,...
                        'HorizontalAlignment','center',...
                        'Position',...
                        [framePixels+7*dx+x1+x2+x3+x4+x5+x6,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x7,...
                         y1],...
                        'String',unitStr,'SelectionHighlight','off', ...
                        'Callback',['ps' psGroup 'Init(''changeUnitClbck'');'],...
                        'TooltipString',toolTipString); 
         otherwise 
          h = warndlg(['Not a valid type: ''' varType ''''],'!! Warning !!');
          set(h,'WindowStyle','modal');
          uiwait(h);
        end
      end
      % If there are multiple variants, then draw the variant stuff
      if psInitStruct.variantHandling.numVariants 
        % The Textbox
        toolTipString = 'Select your favourite!';
        uicontrol(mainFig,'Style','text',...
                  'Units','pixels',...
                  'HorizontalAlignment','right',...
                  'Position',...
                  [framePartSizeX-framePixels-2*dx-x9-x8,...
                   framePixels+dy6+3*y2+2*dy5+dy4,...
                   x8,...
                   y1],...
                  'String','Predefined Variants:',...
                  'SelectionHighlight','off','TooltipString',toolTipString);
        % And the popup
        psInitStruct.uihandleStruct.variantSelector=...
            uicontrol(mainFig,'Style','popupmenu',...
                  'Units','pixels',...
                  'HorizontalAlignment','center',...
                  'Position',...
                  [framePartSizeX-framePixels-dx-x9,...
                   framePixels+dy6+3*y2+2*dy5+dy4,...
                   x9,...
                   y1],...
                   'Value',psInitStruct.variantHandling.selectedVariant,...
                   'Callback',['ps' psGroup 'Init(''changeVariantClbck'');'],...
                  'String',psInitStruct.variantHandling.variantNames,...
                  'SelectionHighlight','off','TooltipString',toolTipString);
      end
      
      % 'Reset' button
      toolTipString = ['Click to reset data for ' psGroup ' (' psSubGroup ')'];
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+dx,...
                 framePixels+dy6+2*y2+2*dy5,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Reset data',...
                'Callback',['ps' psGroup 'Init(''initData'');' ...
                          'ps' psGroup 'Init(''createWindow'');'],...
                'SelectionHighlight','off','TooltipString',toolTipString);

      % 'Load data' button
      toolTipString = 'Click to load parameters from file.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+dx,...
                 framePixels+dy6+y2+dy5,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Load model data',...
                'Callback',['ps' psGroup 'Init(''loadDataClbck'');'],...
                'SelectionHighlight','off','TooltipString',toolTipString);
      % Save data
      toolTipString = 'Click to save current parameters to file.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+2*dx+(framePartSizeX-2*framePixels-3*dx)/2,...
                 framePixels+dy6+y2+dy5,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Save model data',...
                'Callback',['ps' psGroup 'Init(''saveDataClbck'');'],...
                'SelectionHighlight','off','TooltipString',toolTipString);

      % Create 'Done' button.
      toolTipString = 'Click to initialize and exit.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+2*dx+(framePartSizeX-2*framePixels-3*dx)/2,...
                 framePixels+dy6,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Done',...
                'Callback',['ps' psGroup 'Init(''doneButtonClbck'');'],...
                'SelectionHighlight','off','TooltipString',toolTipString);

      % Create 'Cancel' button.
      toolTipString = 'Click to exit without Changing parameters.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+dx,...
                 framePixels+dy6,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Cancel',...
                'Callback',['ps' psGroup 'Init(''cancelButtonClbck'',' num2str(mainFig) ');'],...
                'SelectionHighlight','off','TooltipString',toolTipString);

      if graphPartSizeX > 0
        axesLeft   = (framePartSizeX+axesPixels)/windowSizeX;
        axesWidth  = (graphPartSizeX-2*axesPixels)/windowSizeX;
        if windowSizeY-2*axesPixels < graphPartMaxSizeY 
          tempHeight = windowSizeY-2*axesPixels;
        else
          tempHeight = graphPartMaxSizeY;
        end
        axesHeight = (tempHeight)/windowSizeX;
        axesBottom = (1-axesHeight)/2;
        axesHandle = axes('position',...
                          [axesLeft axesBottom axesWidth axesHeight]);
        psInitStruct.uihandleStruct.plotAxes = axesHandle;
        cla(axesHandle);
        eval(['ps' psGroup 'Init(''generatePlot'',axesHandle);']);
      end
      
      set(mainFig,'WindowStyle','modal')
    end
  end
  % -------------------------------------------------------------------------
  % End Method: createWindow
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : updateWindow
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Updates the window. For now it only recreates the window.
  % -------------------------------------------------------------------------
  if strcmp(method,'updateWindow') 
    
    % Update main windows value boxes.
    updateList = fieldnames(psInitStruct.uihandleStruct.variableList);
    for kk = 1:length(updateList)
      varName  = updateList{kk};
      varValue = psInitStruct.varStruct.(varName);
      ii = find(strcmp(psInitStruct.varStruct.varInfo.varNames,varName));
      unIdx = psInitStruct.varStruct.varInfo.displayedUnitIdx{ii};
      varType = psInitStruct.varStruct.varInfo.varTypes{ii};
      % Manipulate the value to a fitting string
      switch varType
       case 'table'
        if isnumeric(varValue) && varValue == 0
          varStr = [varName ' table NOT loaded...'];            
        else 
          varStr = [varName ' table loaded...'];
        end
       case 'string'
        if ~isstr(varValue) 
          varStr = [varName ' not Found...'];            
        else 
          varStr = varValue;
        end
       case 'value'
        tmpFactor = ...
            psInitStruct.varStruct.varInfo.unitConversionFactors{ii}{unIdx};
        varStr = num2str(varValue/tmpFactor);
      end
      
      set(psInitStruct.uihandleStruct.variableList.(varName).value,...
          'String',varStr)            
      
      if find(strcmp(psInitStruct.varStruct.varInfo.freeVarNames,varName))
          theColour = freeColor;
      else
          theColour = uneditableColor;
      end

      
      if (isfield(psInitStruct.uihandleStruct.variableList.(varName),'opt'))
          optimIsOn = get(psInitStruct.uihandleStruct.variableList.(varName).opt,'Value');
          optimVarIdx = ...
              find(strcmp(psInitStruct.varStruct.varInfo.optimVarNames,varName));
          upLim = psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2};
          lowLim = psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1};
          if strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'abs')
              upLimFinal = upLim/tmpFactor;
              lowLimFinal = lowLim/tmpFactor;
          else
            if varValue
              upLimFinal = upLim/varValue;
              lowLimFinal = lowLim/varValue;
            else
              upLimFinal = NaN;
              lowLimFinal = NaN;
            end
          end
          upLimStr  = num2str(upLimFinal);
          lowLimStr = num2str(lowLimFinal);

          set(psInitStruct.uihandleStruct.variableList.(varName).uplim,...
              'String',upLimStr,'ForegroundColor',0.5*(1+theColour-optimIsOn*(1-theColour)))            
          set(psInitStruct.uihandleStruct.variableList.(varName).lowlim,...
              'String',lowLimStr,'ForegroundColor',0.5*(1+theColour-optimIsOn*(1-theColour)))            
          set(psInitStruct.uihandleStruct.variableList.(varName).limtype,...
              'Value',1+strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'rel'),...
              'ForegroundColor',0.5*(1+theColour-optimIsOn*(1-theColour)))            
      end
    end

    % Check if something has changed and plot it 
    if isempty(psInitStruct.callbackHandling.recentChangedVars)
      recentChangedVars = [];
    else 
      recentChangedVars = ...
          fieldnames(psInitStruct.callbackHandling.recentChangedVars);
    end    
    if graphPartSizeX > 0 && ...
       ~isempty(recentChangedVars) && ...
       ~isempty(plotDependentPars)
      [A , B] = ...
          meshgrid(1:length(recentChangedVars),...
                   1:length(plotDependentPars));
      idxList1 = reshape(A,1,[]);
      idxList2 = reshape(B,1,[]);
      str1 = {recentChangedVars{idxList1}};
      str2 = {plotDependentPars{idxList2}};
      tmpVl=strcmp(str1,str2);
      if ~isempty(find(tmpVl))
        axesHandle = psInitStruct.uihandleStruct.plotAxes;
        cla(axesHandle)
        eval(['ps' psGroup 'Init(''generatePlot'',axesHandle);']);
      end
    end
  end
  % -------------------------------------------------------------------------
  % End Method: updateWindow
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : generatePlot
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- axesHandle
  % -------------------------------------------------------------------------
  % Description : Plots information in the specified axes
  % -------------------------------------------------------------------------
  if strcmp(method,'generatePlot') 
    if nargin < 2
      outValue = -1; 
      disp('You should specify axesHandle to plotfunction.')
      return;
    end
    axesHandle = arg1;    
    axes(axesHandle);    
    title(['ps' psGroups ' function (previev)']);
    xlabel('X value [-]');
    ylabel('Y value [-]');
  end
  % -------------------------------------------------------------------------
  % End Method: generatePlot
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : setParameters
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psSimStruct.psXXX
  % -------------------------------------------------------------------------
  % Description : Set parameters for sub-module to arg1.
  % -------------------------------------------------------------------------
  if strcmp(method, 'setParameters')
    if nargin < 2
      outValue = -1; 
      disp('You should specify new parameters.')
      return;
    end
    newFuncPar = arg1;
    psInitStruct.varStruct = newFuncPar;
    eval(['ps' psGroup 'Init(''createWindow'');']);
  end  
  % -------------------------------------------------------------------------
  % End Method: setParameters
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : saveDataClbck
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- fileName (optional)
  % -------------------------------------------------------------------------
  % Description : Saves present/current values to file
  % -------------------------------------------------------------------------
  if strcmp(method, 'saveDataClbck') 
    eval(['ps' psGroup 'Data = psInitStruct.varStruct;'])

    if nargin < 2      
      [fileName,dirName]=uiputfile('*.mat',['Save ps' psGroup ' data to file']);
    
    else
      fileName = arg1;
      dirName  = '';
    end
    
    if fileName~=0
      try 
        matlabVer = version;
        if str2num(matlabVer(1)) > 6
          save([dirName fileName], ['ps' psGroup 'Data'],'-V6')
        else  
          save([dirName fileName], ['ps' psGroup 'Data'])
        end 
      catch
      button = warndlg(['Could not save to file: ' fileName],['Setup ps' psGroup']);
        uiwait(button);
      end
    else 
      warndlg('Could not save file', ['Setup ps' psGroup]);
    end      
  end
  % -------------------------------------------------------------------------
  % End Method: saveDataClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : doneButtonClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Ends the session by clearing the window, setting
  %               the variables etc.
  % -------------------------------------------------------------------------
  if strcmp(method, 'doneButtonClbck')
    % Clear the psXXX structure.
    psSimStruct.(['ps' psGroup]) = [];
    
    % Set variables needed for simulation
    switch methodType
     case 'createLocal'
      % In case we are a createLocal function then we should
      % manipulate psFuncPar to a local version. Then we should get
      % all these parameters out here and pass them as arguments, since
      % we want to be able to use the 'createFcns' function without the gui.

      psFuncPar = psInitStruct.varStruct;
      psFuncPar.varInfo.origin = psSubGroup;

      psFuncParLocal = psStripToLocalData(psFuncPar);
      
      % Add special stuff. I.e. add 'S' to the variable list.
      extraVars = {'S','B'};
      for kk = 1:length(extraVars)
	psFuncParLocal.(extraVars{kk}) = psFuncPar.(extraVars{kk});
	theIdx = 0;
	for ii = 1:length(psFuncPar.varInfo.varNames)
	  if strcmp(psFuncPar.varInfo.varNames{ii},extraVars{kk})
	    theIdx = ii;
	  end
	end      
	psFuncParLocal.varInfo.varNames{end+1}= ...
	    psFuncPar.varInfo.varNames{theIdx};
	psFuncParLocal.varInfo.varTypes{end+1}= ...
	    psFuncPar.varInfo.varTypes{theIdx};
	psFuncParLocal.varInfo.units{end+1}= ...
	    psFuncPar.varInfo.units{theIdx};
	psFuncParLocal.varInfo.unitConversionFactors{end+1}= ...
	    psFuncPar.varInfo.unitConversionFactors{theIdx};
	psFuncParLocal.varInfo.displayedUnitIdx{end+1}= ...
	    psFuncPar.varInfo.displayedUnitIdx{theIdx};
	psFuncParLocal.varInfo.varDesc{end+1}= ...
	    psFuncPar.varInfo.varDesc{theIdx};
      end
      psFuncParLocal = psCalcDepVars(psFuncParLocal);
      
      % Create the functions
      workingDir = psInitStruct.initSettings.workingDir;
      eval(['ps' psGroup 'Init(''createFcns'',psFuncPar,psFuncParLocal, workingDir);']);
      
      % Set the signal/message-variable
      psSimStruct.(['ps' psGroup]) = psFuncParLocal;
      psSimStruct.(['ps' psGroup]).varInfo.initialized = 1;
      psSimStruct.(['ps' psGroup]).varInfo.localCreated = createsLocal;
     otherwise 
      psSimStruct.(['ps' psGroup]) = psInitStruct.varStruct;
      psSimStruct.(['ps' psGroup]).varInfo.initialized = 1;
      psSimStruct.(['ps' psGroup]).varInfo.localCreated = createsLocal;
    end
    
    % Clear out function-stuff.
    mainFig = psInitStruct.uihandleStruct.mainFig;
    if mainFig 
      delete(mainFig);
    end    
    psInitStruct = [];    
  
  end
  % -------------------------------------------------------------------------
  % End Method: doneButtonClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : cancelButtonClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Ends the session by clearing the window, setting
  %               the variables to their original values etc.
  % -------------------------------------------------------------------------
  if strcmp(method, 'cancelButtonClbck')
    try
        % Set the psXXX structure to its orignal value
        psSimStruct.(['ps' psGroup]) = psInitStruct.initSettings.originalSimStruct;
        
        % Clear out function-stuff.
        mainFig = psInitStruct.uihandleStruct.mainFig;
        if mainFig 
            delete(mainFig);
        end    
        psInitStruct = [];
        psStartUp('updateMainWindow');
    catch 
        button = warndlg(['Couln''t restore original simstruct because of an internal error in ps' psGroup '.'],['Setup ps' psGroup]);
        uiwait(button);
        delete(arg1);
        psStartUp('updateMainWindow');
    end
  end
  % -------------------------------------------------------------------------
  % End Method: cancelButtonClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : loadDataClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Loads data from file
  % -------------------------------------------------------------------------  
  if strcmp(method, 'loadDataClbck') 
    if nargin < 2      
      [fileName,dirName]=uigetfile('*.mat',['Load ps' psGroup ' data from file']);
    else 
      fileName = arg1;
      dirName = '';
    end
    
    oldFuncPar = psInitStruct.varStruct;
    if fileName~=0
      try 
        parameters= load([dirName fileName]);
        psInitStruct.varStruct = parameters.(['ps' psGroup 'Data']);
        % Just to see that they are all there
        tags = psInitStruct.varStruct.varInfo.varNames;
        for i = 1:length(tags)
          temp = psInitStruct.varStruct.(tags{i});
        end
        isRightOrigin=strcmp(psInitStruct.varStruct.varInfo.origin,...
                               psSubGroup );
        isRightVersion=strcmp(psInitStruct.varStruct.varInfo.psPackVersion,...
                              psPackVersion);
        if ~isRightOrigin || ~isRightVersion
          h = warndlg({['There is a mismatch between origin name ',...
                        'and or version of the creator of the data. ',...
                        'This may, or may not, be a problem. ',...
                        'You should however consider recreating the ',...
                        'data.'],'',...
                       'Another possibility is to manually load the ',...
                       'data file and modify version/origin. (After ',...
                       'checking that your file is correct.)'},...
                      'psPack (Warning)');
        else
          eval(['ps' psGroup 'Init(''createWindow'');']);
        end
      catch
        button = warndlg(['Could not load file: ' fileName],['Setup ps' psGroup]);
        psInitStruct.varStruct=oldFuncPar;
        uiwait(button);
      end
    else
      button =warndlg('Could not load file',['Setup ps' psGroup]);
      psInitStruct.varStruct=oldFuncPar;
      uiwait(button);
    end
  end
  % -------------------------------------------------------------------------
  % End Method: loadDataClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : changeValClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Executes operations necessary after a change of a variable
  % -------------------------------------------------------------------------
  if strcmp(method, 'changeValClbck')
    tags = psInitStruct.varStruct.varInfo.varNames;
    for ii = 1:length(tags)
     varTypeIdx=find(strcmp(psInitStruct.varStruct.varInfo.varNames,tags{ii}));
     varType = psInitStruct.varStruct.varInfo.varTypes{varTypeIdx};

     optimVarIdx = ...
         find(strcmp(psInitStruct.varStruct.varInfo.optimVarNames,tags{ii}));
     switch varType
       case 'table'
        % Do nothinh
       case 'string'
        oldValue{ii} = psInitStruct.varStruct.(tags{ii});
        psInitStruct.varStruct.(tags{ii})= ...
      get(psInitStruct.uihandleStruct.variableList.(tags{ii}).value, 'String');
       case 'value'
        oldValue{ii} = psInitStruct.varStruct.(tags{ii});
       tmpIdx = find(strcmp(psInitStruct.varStruct.varInfo.varNames,tags{ii}));
        unIdx = psInitStruct.varStruct.varInfo.displayedUnitIdx{tmpIdx};
        tmpFactor = ...
           psInitStruct.varStruct.varInfo.unitConversionFactors{tmpIdx}{unIdx};
        tmpValue = ...
      str2num(get(psInitStruct.uihandleStruct.variableList.(tags{ii}).value,...
                  'String'));
        if ~isempty(tmpValue) && ~any(isnan(tmpValue))
            psInitStruct.varStruct.(tags{ii})= tmpValue(1)*tmpFactor;
        end        
        if ~isempty(optimVarIdx)
          oldLimValues{optimVarIdx} = ...
              psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx};
          tmpLowLim = str2num(get(psInitStruct.uihandleStruct.variableList.(tags{ii}).lowlim,'String'));
          tmpUpLim  = str2num(get(psInitStruct.uihandleStruct.variableList.(tags{ii}).uplim,'String'));
          if ~isempty(tmpLowLim)
              if strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'abs')
                  tmpLowLim = tmpLowLim(1)*tmpFactor;
              else
                  tmpLowLim = tmpLowLim(1)*oldValue{ii};
              end
              psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1} = tmpLowLim;
          end        
          if ~isempty(tmpUpLim)
              if strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'abs')
                  tmpUpLim = tmpUpLim(1)*tmpFactor;
              else
                  tmpUpLim = tmpUpLim(1)*oldValue{ii};
              end
              psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2} = tmpUpLim;
          end        
          theStr = get(psInitStruct.uihandleStruct.variableList.(tags{ii}).limtype,'String');
          psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx} = ...
              theStr{get(psInitStruct.uihandleStruct.variableList.(tags{ii}).limtype,'Value')}; 
        end
     end      
    end

    % Calculate and set the non-free-variables.
    psInitStruct.varStruct = psCalcDepVars(psInitStruct.varStruct);

    % Now check what parameters has really changed and list them
    psInitStruct.callbackHandling.recentChangedVars = [];
    tags = psInitStruct.varStruct.varInfo.varNames;
    for ii = 1:length(tags)
      theVarType = psInitStruct.varStruct.varInfo.varTypes{ii};
      switch theVarType
       case 'value'
         % Since stuff may change due to numerical problems we only
         % consider a value to have changed if a value changes more
         % than 1e-6 relatively. 
         if ~any(isnan(psInitStruct.varStruct.(tags{ii}))) && all(psInitStruct.varStruct.(tags{ii}))
             if any(abs(oldValue{ii}./psInitStruct.varStruct.(tags{ii})-1) > 1e-6)
                 psInitStruct.callbackHandling.recentChangedVars.(tags{ii}) = 1;
             end
         else
             if oldValue{ii} ~= 0
                 psInitStruct.callbackHandling.recentChangedVars.(tags{ii}) = 1;
             end
         end
         optimVarIdx = ...
             find(strcmp(psInitStruct.varStruct.varInfo.optimVarNames,tags{ii}));
         if ~isempty(optimVarIdx)
             newIsComparable = ~any(isnan(psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1})) && ...
                 all(psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1});
             if (newIsComparable && any(abs(oldLimValues{optimVarIdx}{1}./ ...
                                        psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1}-1) <= 1e-6)) || ...
                     any(isnan(psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1}))
                 psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1}=oldLimValues{optimVarIdx}{1};
             end
             newIsComparable = ~any(isnan(psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2})) && ...
                 all(psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2});
             if (newIsComparable && any(abs(oldLimValues{optimVarIdx}{2}./ ...
                                        psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2}-1) <= 1e-6))  || ...
                     any(isnan(psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2}))
                 psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2}=oldLimValues{optimVarIdx}{2};
             end
             % If a variables value has changed we may update optimLimits
             % if it set to be relative so that it stays relative
             if isfield(psInitStruct.callbackHandling.recentChangedVars,tags{ii}) && ...
                     psInitStruct.callbackHandling.recentChangedVars.(tags{ii})==1  && ...
                     strcmp(psInitStruct.varStruct.varInfo.optimShownInfo{optimVarIdx},'rel') && ...
                     all(oldValue{ii})
                 lowLimRel = psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1}./oldValue{ii};
                 upLimRel = psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2}./oldValue{ii};                 
                 psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{1}=psInitStruct.varStruct.(tags{ii})*lowLimRel;
                 psInitStruct.varStruct.varInfo.optimLimits{optimVarIdx}{2}=psInitStruct.varStruct.(tags{ii})*upLimRel;
                 
             end
         end
       case 'table'
        % Do nothing
       case 'string'
        % Unlikely but possibly do we want to update plot when a
        % string value changes.
        if strcmp(oldValue{ii},psInitStruct.varStruct.(tags{ii}))
          psInitStruct.callbackHandling.recentChangedVars.(tags{ii}) = 1;
        end
       otherwise
        % Should complain...
      end
    end
    eval(['ps' psGroup 'Init(''updateWindow'');'])
  end
  % -------------------------------------------------------------------------
  % End Method: changeValClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : changeOptClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Executes operations needed after a opt-box-click
  % -------------------------------------------------------------------------
  if strcmp(method, 'changeOptClbck')
    tags = psInitStruct.varStruct.varInfo.optimVarNames;
    for ii = 1:length(tags)
      % Now, the only thing that is needed is a translation from
      % value to string
      if get(psInitStruct.uihandleStruct.variableList.(tags{ii}).opt,'Value')
        psInitStruct.varStruct.varInfo.optimCtrl{ii} = 'on';
        %disp('on')
      else
        psInitStruct.varStruct.varInfo.optimCtrl{ii} = 'off';
        %disp('off')
      end
    end    
      % Need to update the window i should think
      eval(['ps' psGroup 'Init(''updateWindow'');'])
  end  
  % -------------------------------------------------------------------------
  % End Method: changeOptClbck
  % -------------------------------------------------------------------------  

  % -------------------------------------------------------------------------
  % Method      : changeUnitClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Executes operations needed after a unitchange
  % -------------------------------------------------------------------------
  if strcmp(method,'changeUnitClbck')
    % Sort out which fieldnames there are in the figpar
    tmpFieldNames = fieldnames(psInitStruct.uihandleStruct.variableList);
    for ii = 1:length(tmpFieldNames)
      varName = tmpFieldNames{ii};
      % Now if it is a unit that is changeable ... 
   if isfield(psInitStruct.uihandleStruct.variableList.(varName),'unit') && ...
strcmp(get(psInitStruct.uihandleStruct.variableList.(varName).unit,'Style'),...
                   'popupmenu')
        % ... we should sort out what idx the variable has and ...
        theIdx = find(strcmp(psInitStruct.varStruct.varInfo.varNames,varName));
        % ... set the variables displayedUnitIdx.
        psInitStruct.varStruct.varInfo.displayedUnitIdx{theIdx} =...
          get(psInitStruct.uihandleStruct.variableList.(varName).unit,'Value');
      end      
    end    
    
    eval(['ps' psGroup 'Init(''updateWindow'');'])
  end  
  % -------------------------------------------------------------------------
  % End Method: changeUnitClbck
  % -------------------------------------------------------------------------  
  
  % -------------------------------------------------------------------------
  % Method      : changeVariantClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Performs actions needed when another variant of
  %               intial data is selected
  % ------------------------------------------------------------------------- 
  if strcmp(method,'changeVariantClbck')
    oldVariant = psInitStruct.variantHandling.selectedVariant;
    psInitStruct.variantHandling.selectedVariant = ...
        get(psInitStruct.uihandleStruct.variantSelector,'Value');
    if psInitStruct.variantHandling.selectedVariant ~= oldVariant
      eval(['ps' psGroup 'Init(''initData'');'])
      eval(['ps' psGroup 'Init(''createWindow'');']) % Relly needs to re-create
                                                     % the window since the layout
                                                     % might have changed.
    end
  end
  % -------------------------------------------------------------------------
  % End Method: changeVariantClbck
  % -------------------------------------------------------------------------
    
  % -------------------------------------------------------------------------
  % Method      : createFcns
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psSimStruct.psXXX
  %               arg2 <- psVarStructLocal
  %               arg3 <- workingDir 
  % -------------------------------------------------------------------------
  % Description : Creates local functions for sub-module
  % -------------------------------------------------------------------------
  if strcmp(method,'createFcns')  
    if nargin < 4
      outValue = -1; 
      disp(['You should specify psSimStruct.ps' psGroup ', psVarStructLocal ' ...
            'and workingDir']) 
      return;
    end
    
    varStruct = arg1;
    psVarStructLocal = arg2;
    workingDir = arg3;
    
    functionNames={'psVolume.m','psDVolume.m','psArea.m'};
    FID=fopen([workingDir filesep functionNames{1}],'wt');
    if FID==0,
      error(['Could not open the ' functionNames{1}]);
    end
    fprintf(FID,'function [vol]=psVolume(theta);\n\n');
    fprintf(FID,'%% [vol]=psVolume(theta)\n');
    fprintf(FID,'%% Calculates the cylinder volume for the crank angle theta, and adds\n');
    fprintf(FID,'%% the clearance volume v_c from the global variable.\n%% \n');
    fprintf(FID,'%% Warning do not edit this function it is generated by psSetupVolume\n');
    fprintf(FID,'%% and will be over written when a new configuration is defined.\n\n');
    fprintf(FID,'global psSimStruct;\n \n \n');
    fprintf(FID,'s= %f*cos(theta) + sqrt(%f - %f*sin(theta).^2);\n',varStruct.a,varStruct.l^2,varStruct.a^2);
    fprintf(FID,'vol=psSimStruct.psGeometry.V_c + %f *( %f - s);\n',pi*varStruct.B^2/4,varStruct.l+varStruct.a);
    fclose(FID);
    disp(['Successfully created ''' functionNames{1} '''.']);
    
    FID=fopen([workingDir filesep functionNames{2}],'wt');
    if FID==0,
      error(['Could not open the ' functionNames{2}]);
    end
    fprintf(FID,'function [dVol]=psDVolume(theta);\n\n');
    fprintf(FID,'%% [vol]=psDVolume(theta)\n');
    fprintf(FID,'%% Calculates the crank angle derivative of the cylinder volume\n');
    fprintf(FID,'%% for the crank angle theta.\n%% \n');
    fprintf(FID,'%% Warning do not edit this function it is generated by fixParGeometry/psGeometryInit\n');
    fprintf(FID,'%% and will be over written when a new configuration is defined.\n\n');
    fprintf(FID,'s= sin(theta).*(%f + %f.*cos(theta)./sqrt(%f - %f*sin(theta).^2));\n',varStruct.a,varStruct.a^2,varStruct.l^2,varStruct.a^2);
    fprintf(FID,'dVol=%f*s;\n',pi*varStruct.B^2/4);
    fclose(FID);
    disp(['Successfully created ''' functionNames{2} '''.']);
    
    FID=fopen([workingDir filesep functionNames{3}],'wt');
    if FID==0,
      error(['Could not open the ' functionNames{3}]);
    end
    fprintf(FID,'function [area]=psArea(theta);\n\n');
    fprintf(FID,'%% [area]=psArea(theta)\n');
    fprintf(FID,'%% Calculates the cylinder and piston area exposed to heat transfer\n');
    fprintf(FID,'%% for the crank angle theta. The cylinder liner area above the position\n');
    fprintf(FID,'%% when the piston is at TDC is neglected.\n');
    fprintf(FID,'%% \n');
    fprintf(FID,'%% Warning do not edit this function it is generated by fixParGeometry/psGeometryInit\n');
    fprintf(FID,'%% and will be over written when a new configuration is defined.\n\n');
    fprintf(FID,'s= %f*cos(theta) + sqrt(%f - %f*sin(theta).^2);\n',varStruct.a,varStruct.l^2,varStruct.a^2);
    fprintf(FID,'area=%f + %f *( %f - s);\n',pi*varStruct.B^2/2,pi*varStruct.B,varStruct.l+varStruct.a);
    fclose(FID);
    disp(['Successfully created ''' functionNames{3} '''.']);
    
    %% psXXXInit.m
    functionName='psGeometryInit.m';
    FID=fopen([workingDir filesep functionName],'wt');
    if FID <=0
      error(['Could not open the ' functionName]);
    end

    fprintf(FID,'function outValue = psGeometryInit(method,arg1,arg2,arg3)\n');
    fprintf(FID,'%% This is almost an empty initfunction. The intention is that it\n');
    fprintf(FID,'%% should only set the initialization-struct values so that psPack\n');
    fprintf(FID,'%% knows that the functionality is initialized. However\n');
    fprintf(FID,'%% it is somewat special since it also sets the psGeometry\n');
    fprintf(FID,'%% parameters in case someone else wants them.\n');
    fprintf(FID,'%%\n');
    fprintf(FID,'%% Warning, this function is automatically generated by\n');
    fprintf(FID,'%% fixParGeometry/psGeometryInit.m. Any modifications will be lost when updated.\n');
    fprintf(FID,'\n');
    fprintf(FID,'global psSimStruct\n');
    fprintf(FID,'\n');
    fprintf(FID,'outValue = 0;\n');
    fprintf(FID,'if strcmp(method,''init'')\n');
    fprintf(FID,'  psSimStruct.psGeometry = [];\n');
    fprintf(FID,'  psGeometryTmp = load(''psGeometryLocalData'');\n');
    fprintf(FID,'  psSimStruct.psGeometry = psGeometryTmp.psGeometry;\n');
    fprintf(FID,'  psSimStruct.psGeometry.varInfo.initialized = 1;\n');

    fprintf(FID,['  isRightOrigin = strcmp(psSimStruct.psGeometry.varInfo.origin,''' [psSubGroup 'Local'] ''') ;\n']);
    fprintf(FID,['  isRightVersion = strcmp(psSimStruct.psGeometry.varInfo.psPackVersion,''' psPackVersion ''') ;\n']);
    fprintf(FID,'  if ~isRightOrigin || ~isRightVersion\n');
    fprintf(FID,'    h = warndlg(''There is a mismatch between origin name and or version of the creator of the data. This may, or may not, be a problem. You should however consider reinitializing/recreating the data.'',''psPack (Warning)''); \n');
    fprintf(FID,'    set(h,''WindowStyle'',''modal'');\n');
    fprintf(FID,'    uiwait(h);\n');
    fprintf(FID,'  end\n');
    fprintf(FID,'  \n');
    fprintf(FID,'  outValue = 0;\n');
    fprintf(FID,'end\n');
    fprintf(FID,'\n');
    fprintf(FID,'\n');
    fprintf(FID,'if strcmp(method,''checkType'')\n');
    fprintf(FID,'  outValue = ''localFile'';\n');
    fprintf(FID,'end\n');
    fclose(FID);
    disp(['Successfully created ''' functionName '''.']);
    
    %% psXXXReadme.txt
    functionName='psGeometryReadme.txt';
    FID=fopen([workingDir filesep functionName],'wt');
    if FID<=0
      error(['Could not open the ' functionName]);
    end
    fprintf(FID,'With this choice of geometry files \n');
    fprintf(FID,'the geometry is fixed within \n');
    fprintf(FID,'the generated files. Some paramaters \n');
    fprintf(FID,'are adjustable however. Read psArea.m, \n');
    fprintf(FID,'psVolume.m and psDVolume.m to se which.\n');
    fprintf(FID,'\n');
    fprintf(FID,'V_c,A_th and C_d are \n');
    fprintf(FID,'adjustable parameters.\n');

    fclose(FID);
    disp(['Successfully created ''' functionName '''.']);

    % psXXXLocalData.mat
    psGeometry = psVarStructLocal;
    psGeometry.varInfo.psPackVersion = psPackVersion;
    matlabVer = version;
    if str2num(matlabVer(1)) > 6
      save([workingDir filesep 'psGeometryLocalData'],'psGeometry','-V6')
    else  
      save([workingDir filesep 'psGeometryLocalData'],'psGeometry','-V6')
    end 
    disp(['Successfully created ''' 'psGeometryLocalData.mat' '''.']);
        
    % psXXXFiles.mat
    psGeometryFiles = {'psArea.m';...
                       'psVolume.m';...
                       'psDVolume.m';...
                       'psGeometryInit.m';...
                       'psGeometryReadme.txt';...
                       'psGeometryFiles.mat';...
                       'psGeometryLocalData.mat'};
    matlabVer = version;
    if str2num(matlabVer(1)) > 6
      save([workingDir filesep 'psGeometryFiles'], ...
           'psGeometryFiles','-V6')
    else  
      save([workingDir filesep 'psGeometryFiles'], ...
           'psGeometryFiles')
    end 
    disp(['Successfully created ''' 'psGeometryFiles.mat' '''.']);
  end
  % -------------------------------------------------------------------------
  % End Method: createFcns
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : checkType
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Returns the type of the psXXXInit.m
  % -------------------------------------------------------------------------
  if strcmp(method,'checkType')
    outValue = methodType;
  end  
  % -------------------------------------------------------------------------
  % End Method: checkType
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : getInitStruct
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Return the initStruct. (For debugging purposes.)
  % -------------------------------------------------------------------------
  if strcmp(method,'getInitStruct')
    outValue = psInitStruct;
  end  
  % -------------------------------------------------------------------------
  % End Method: getInitStruct
  % -------------------------------------------------------------------------
