%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

% psPack not really needed for this function
% - addpath /home/oberg/Work/psPack_WC_git
% - psPack

%cheppPath = '/home/oberg/temp/Chepp/';
%cheppPath = '/home/oberg/Work/Chepp/';
%addpath(cheppPath)


%chepp

theFuel = 'Isooctane';

global FuelData
global b_VECTOR
global MOLEC_WEIGHT
global SELECTED_NAMES
R_tilde=8.3143;

fnamesU = {'Mf', 'cpf', 'hf', 'Ma', 'cpa', 'ha', 'T'};
fnamesB = {'cpb','cvb','hb','Mb','dubdp', 'dRbdp','dRbdT','dubdxf','dRbdxf','dubdxa','dRbdxa','phi','T','p','xf'};

gammaApprox_temp=500;
gammaApprox_p=2e5;

fuelIndx = find (strcmp(FuelData.longname, theFuel));
if fuelIndx ~= 0
    cheppGui('FuelRadioPush',fuelIndx)
    % Use the standard species
    cheppGui('CompileData')

    setPhi(1)
    Weights = [0.0120 ; 0.0010 ; 0.0160 ; 0.0140];
    AFs = sum(b_VECTOR(3:4).*Weights(3:4))/sum(b_VECTOR(1:2).*Weights(1:2));
    chemPropB.T   = (250:50:3500);
    %chemPropB.T   = (250:12.5:3500);
    %chemPropB.T   = (250:5:3500);
    %chemPropB.T   = (250:25:3500);
    %chemPropB.T   = (500:500:3500);
    %chemPropB.p   = [2e4:(1e4/2):9.5e4 1e5:(1e5)/2:19.5e5 2e6:(25e4/2):387.5e4 4e6:5e5/2:97.5e5 ...
    %                 1e7:1e6/2:2e7];
    chemPropB.p   = [2e4:1e4:9e4 1e5:1e5:19e5 2e6:25e4:375e4 4e6:5e5:95e5 ...
                     1e7:1e6:2e7];
    %chemPropB.p   = [1e4 1e5 1e6];
    phi_max = 3.12;
    xf_max = 1/(1+AFs/phi_max);    
    chemPropB.xf  = 0:(xf_max)/39:xf_max;
    %chemPropB.xf =1/(AFs+1);
    %chemPropB.phi = 1;
    chemPropB.phi = chemPropB.xf*AFs./(1-chemPropB.xf);
    %chemPropB.phi = [0 1e-6 0.1:0.1:3];
    %chemPropB.phi = 0.2:0.25:2;
    calcStart = clock;
    w8handle = waitbar(0,'Progress - ETA: ');
    totalIters = length(chemPropB.T)*length(chemPropB.p);
    theIter = 0;
    for kk = 1:length(chemPropB.phi)
%        if chemPropB.phi(kk) == 0
%             for jj = 1:length(chemPropB.p)
%                 for ii = 1:length(chemPropB.T)
%                     [cp_tilde,h_tilde,dummy,M_tilde]= ...
%                         thermPropAir(chemPropB.T(ii));
%                     chemPropB.cpb(ii,jj,kk) = cp_tilde/M_tilde;
%                     chemPropB.cvb(ii,jj,kk) = (cp_tilde-R_tilde)/M_tilde;
%                     chemPropB.hb(ii,jj,kk) = h_tilde/M_tilde;
%                     chemPropB.Mb(ii,jj,kk) = M_tilde;
%                     chemPropB.dhbdp(ii,jj,kk) = 0;
%                     chemPropB.dRbdp(ii,jj,kk) = 0;
%                     chemPropB.dRbdT(ii,jj,kk) = 0;
%                 end
%             end
        if chemPropB.phi(kk) <= 1e-6
            setPhi(1e-6)
        else
            setPhi(chemPropB.phi(kk))
        end
        for jj = 1:length(chemPropB.p)
            theIter = theIter + 1;
            %theIter = (ii-1)*length(chemPropB.phi)+jj;
            xready = theIter/totalIters;
            calcETA = round(etime(clock,calcStart)*(totalIters/theIter-1));
            w8handle = waitbar(xready);
            waitbar(xready,w8handle,sprintf('Progress - ETA: %s s', ...
                                            num2str(calcETA))); 
            for ii = 1:length(chemPropB.T)
                
                T = chemPropB.T(ii);
                p = chemPropB.p(jj);
                phi = chemPropB.phi(kk);
                xf  = chemPropB.xf(kk);
                if ii == 1 % Do it twice for every start temperature
                    chemEqSolve(T,p);
                end
                T_frozen = 1700;
                p_frozen = 1e5;
                if T >  T_frozen
                    [X,dXdT,dXdp,dXdb] = chemEqSolve(T,p);
                else
                    [X,dXdT,dXdp,dXdb] = chemEqSolve(T_frozen,p_frozen);
                    dXdT = zeros(size(dXdT));
                    dXdp = zeros(size(dXdp));       
                    % Leave dXdb for now
                end
                
                [cp_tilde_all,h_tilde_all] = thermPropAll(T);

                dbdphi=cheppGui('dbdphi');

                                
                dXdphi   = dXdb*dbdphi;
                dXdxf    = dXdphi*AFs/(1-xf);
                dXdxa    = dXdphi*(-xf*AFs)/(1-xf)^2;
                M        = MOLEC_WEIGHT*X;
                dMdT     = MOLEC_WEIGHT*dXdT;
                dMdp     = MOLEC_WEIGHT*dXdp;
                dMdphi   = MOLEC_WEIGHT*dXdphi;
                dMdxf    = MOLEC_WEIGHT*dXdxf;
                dMdxa    = MOLEC_WEIGHT*dXdxa;
                R        = R_tilde/M;               
                dRdp     = -R_tilde/M^2*dMdp;
                dRdT     = -R_tilde/M^2*dMdT;
                dRdphi   = -R_tilde/M^2*dMdphi;
                dRdxf    = -R_tilde/M^2*dMdxf;
                dRdxa    = -R_tilde/M^2*dMdxa;
                h        = X'*h_tilde_all/M;
                cp_tilde = X'*cp_tilde_all+dXdT'*h_tilde_all;
                cp       = (cp_tilde-(dMdT*X'*h_tilde_all)/M)/M;
                dhdp     = (h_tilde_all'*dXdp-h*dMdp)/M;
                dhdphi   = (h_tilde_all'*dXdphi-h*dMdphi)/M;
                dhdxf    = (h_tilde_all'*dXdxf-h*dMdxf)/M;
                dhdxa    = (h_tilde_all'*dXdxa-h*dMdxa)/M;
                u        = (X'*h_tilde_all-R_tilde*T)/M; 
                cv_tilde = cp_tilde-R_tilde*T;
                cv       = cp-R+R*T/M*dMdT;
                dudp     = ((h_tilde_all-R_tilde*T)'*dXdp-u*dMdp)/M;
                dudphi   = ((h_tilde_all-R_tilde*T)'*dXdphi-u*dMdphi)/M;
                dudxf    = ((h_tilde_all-R_tilde*T)'*dXdxf-u*dMdxf)/M;
                dudxa    = ((h_tilde_all-R_tilde*T)'*dXdxa-u*dMdxa)/M;
                    
                chemPropB.hb(ii,jj,kk) = h;
                chemPropB.cpb(ii,jj,kk) = cp;
                chemPropB.cvb(ii,jj,kk) = cv;
                chemPropB.Mb(ii,jj,kk) = M;
                chemPropB.dRbdp(ii,jj,kk) = dRdp;
                chemPropB.dRbdT(ii,jj,kk) = dRdT;
                chemPropB.dubdp(ii,jj,kk) = dudp;
                chemPropB.dubdxf(ii,jj,kk) = dudxf;
                chemPropB.dRbdxf(ii,jj,kk) = dRdxf;
                chemPropB.dubdxa(ii,jj,kk) = dudxa;
                chemPropB.dRbdxa(ii,jj,kk) = dRdxa;

                
                if 0
                    % Testing valididy                                
                    T_diff = 300*1e-6;
                    p_diff = 1e5*1e-6;
                    phi_diff = 1*1e-7;
                    phi1=chemPropB.phi(kk)+phi_diff;
                    phi2=chemPropB.phi(kk)-phi_diff;
                    xfa_diff  = (1/(1+AFs/phi1)-1/(1+AFs/phi2))/2;
                    T_plus=chemPropB.T(ii)+T_diff;
                    T_minus=chemPropB.T(ii)-T_diff;
                    X_T_plus = chemEqSolve(chemPropB.T(ii)+T_diff, ...
                                           chemPropB.p(jj)); 
                    [cp_tilde_all_T_plus,h_tilde_all_T_plus] = ...
                        thermPropAll(chemPropB.T(ii)+T_diff); 
                    X_T_minus= chemEqSolve(chemPropB.T(ii)-T_diff, ...
                                           chemPropB.p(jj)); 
                    [cp_tilde_all_T_minus,h_tilde_all_T_minus] = ...
                        thermPropAll(chemPropB.T(ii)-T_diff); 
                    X_p_plus = chemEqSolve(chemPropB.T(ii), ...
                                           chemPropB.p(jj)+p_diff); 
                    X_p_minus= chemEqSolve(chemPropB.T(ii), ...
                                           chemPropB.p(jj)-p_diff); 
                    setPhi(chemPropB.phi(kk)+phi_diff)
                    X_phi_plus = chemEqSolve(chemPropB.T(ii), ...
                                             chemPropB.p(jj)); 
                    X_phi_plus = chemEqSolve(chemPropB.T(ii), ...
                                             chemPropB.p(jj)); 
                    setPhi(chemPropB.phi(kk)-phi_diff)
                    X_phi_minus= chemEqSolve(chemPropB.T(ii), ...
                                             chemPropB.p(jj)); 
                    X_phi_minus= chemEqSolve(chemPropB.T(ii), ...
                                             chemPropB.p(jj)); 
                    setPhi(chemPropB.phi(kk));
                    
                    dMdT_approx = (MOLEC_WEIGHT*X_T_plus-...
                                   MOLEC_WEIGHT*X_T_minus)/(2*T_diff);
                    dMdp_approx = (MOLEC_WEIGHT*X_p_plus- ...
                                   MOLEC_WEIGHT*X_p_minus)/(2*p_diff);

                    dMdphi_approx =(MOLEC_WEIGHT*X_phi_plus- ...
                                    MOLEC_WEIGHT*X_phi_minus)/(2*phi_diff);

                    R_T_plus  = R_tilde/(MOLEC_WEIGHT*X_T_plus);
                    R_T_minus = R_tilde/(MOLEC_WEIGHT*X_T_minus);
                    dRdT_approx = (R_T_plus-R_T_minus)/(2*T_diff);
                    R_p_plus  = R_tilde/(MOLEC_WEIGHT*X_p_plus);
                    R_p_minus = R_tilde/(MOLEC_WEIGHT*X_p_minus);
                    dRdp_approx = (R_p_plus-R_p_minus)/(2*p_diff);
                    R_phi_plus  = R_tilde/(MOLEC_WEIGHT*X_phi_plus);
                    R_phi_minus = R_tilde/(MOLEC_WEIGHT*X_phi_minus);
                    dRdphi_approx = (R_phi_plus-R_phi_minus)/(2*phi_diff);
                    dRdxfa_approx = (R_phi_plus-R_phi_minus)/(2*xfa_diff);
                    h_T_plus = X_T_plus'*h_tilde_all_T_plus/ ...
                        (MOLEC_WEIGHT*X_T_plus); 
                    h_T_minus = X_T_minus'*h_tilde_all_T_minus/ ...
                        (MOLEC_WEIGHT*X_T_minus); 
                    cp_approx = (h_T_plus-h_T_minus)/(2*T_diff);
                    h_p_plus=X_p_plus'*h_tilde_all/(MOLEC_WEIGHT* ...
                                                    X_p_plus);
                    h_p_minus = X_p_minus'*h_tilde_all/(MOLEC_WEIGHT* ...
                                                        X_p_minus); 
                    dhdp_approx = (h_p_plus-h_p_minus)/(2*p_diff);
                    h_phi_plus = X_phi_plus'*h_tilde_all/ ...
                        (MOLEC_WEIGHT*X_phi_plus); 
                    h_phi_minus = X_phi_minus'*h_tilde_all/ ...
                        (MOLEC_WEIGHT*X_phi_minus); 
                    dhdphi_approx = (h_phi_plus-h_phi_minus)/(2*phi_diff);
                    
                    u_T_plus   = (X_T_plus'*h_tilde_all_T_plus-...
                                  R_tilde*T_plus)/(MOLEC_WEIGHT*X_T_plus);
                    u_T_minus =(X_T_minus'*h_tilde_all_T_minus- ...
                                R_tilde*T_minus)/(MOLEC_WEIGHT*X_T_minus);
                    cv_approx = (u_T_plus-u_T_minus)/(2*T_diff);
                    u_tilde_all = h_tilde_all-R_tilde*T;
                    u_p_plus=X_p_plus'*u_tilde_all/(MOLEC_WEIGHT*X_p_plus);
                    u_p_minus=X_p_minus'*u_tilde_all/(MOLEC_WEIGHT* ...
                                                      X_p_minus); 
                    dudp_approx = (u_p_plus-u_p_minus)/(2*p_diff);
                    u_phi_plus = X_phi_plus'*u_tilde_all/ ...
                        (MOLEC_WEIGHT*X_phi_plus); 
                    u_phi_minus = X_phi_minus'*u_tilde_all/ ...
                        (MOLEC_WEIGHT*X_phi_minus); 
                    dudphi_approx = (u_phi_plus-u_phi_minus)/(2*phi_diff);
                    dudxfa_approx = (u_phi_plus-u_phi_minus)/(2*xfa_diff);
                    

                    
                    testStruct.dMdT(ii,jj,kk)= dMdT; % 7.4e-14
                    testStruct.dMdT_approx(ii,jj,kk)= dMdT_approx;
                    testStruct.dMdp(ii,jj,kk)= dMdp; % 1.39e-16
                    testStruct.dMdp_approx(ii,jj,kk)= dMdp_approx;%9.43e-4
                    testStruct.dMdphi(ii,jj,kk)= dMdphi;
                    testStruct.dMdphi_approx(ii,jj,kk)= dMdphi_approx;

                    if 0
                        figure;
                        plot(reshape(testStruct.dMdT_approx,1,[]),...
                             reshape(testStruct.dMdT,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dMdp_approx,1,[]),...
                             reshape(testStruct.dMdp,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dMdphi_approx,1,[]),...
                             reshape(testStruct.dMdphi,1,[]),'.')
                        
                        
                        %  figure;
                        %  plot(reshape(testStruct.dMdphi_approx- ...
                        %             testStruct.dMdphi,1,[]),'.') 
                        %  hold on
                        %  plot(reshape(-testStruct.phi/1000,1,[]),'r.')
                    end
                    
                    testStruct.dRdT(ii,jj,kk)= dRdT;
                    testStruct.dRdT_approx(ii,jj,kk)= dRdT_approx;
                    testStruct.dRdp(ii,jj,kk)= dRdp;
                    testStruct.dRdp_approx(ii,jj,kk)= dRdp_approx;
                    testStruct.dRdphi(ii,jj,kk)= dRdphi;  
                    testStruct.dRdphi_approx(ii,jj,kk)= dRdphi_approx; 
                    testStruct.dRdxfa_approx(ii,jj,kk)= dRdxfa_approx;  
                    testStruct.dRdxf(ii,jj,kk)= dRdxf;  
                    testStruct.dRdxa(ii,jj,kk)= dRdxa;  
                    if 0
                        figure;
                        plot(reshape(testStruct.dRdT_approx,1,[]),...
                             reshape(testStruct.dRdT,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dRdp_approx,1,[]),...
                             reshape(testStruct.dRdp,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dRdphi_approx,1,[]),...
                             reshape(testStruct.dRdphi,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dRdxfa_approx,1,[]),...
                             reshape(testStruct.dRdxf-testStruct.dRdxa,1,[]),'.')
                        
                    end
                    
                    testStruct.cp(ii,jj,kk)= cp;
                    testStruct.cp_approx(ii,jj,kk)= cp_approx;
                    testStruct.dhdp(ii,jj,kk)= dhdp;
                    testStruct.dhdp_approx(ii,jj,kk)= dhdp_approx;
                    testStruct.dhdphi(ii,jj,kk)= dhdphi;
                    testStruct.dhdphi_approx(ii,jj,kk)= dhdphi_approx;
                    if 0 
                        figure;
                        plot(reshape(testStruct.cp_approx,1,[]),...
                             reshape(testStruct.cp,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dhdp_approx,1,[]),...
                             reshape(testStruct.dhdp,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dhdphi_approx,1,[]),...
                             reshape(testStruct.dhdphi,1,[]),'.')
                        % figure;
                        % plot(reshape(testStruct.dhdxf_approx,1,[]),...
                        %     reshape(testStruct.dhdxf,1,[]),'.')

                    end
                    
                    testStruct.cv(ii,jj,kk)= cv;
                    testStruct.cv_approx(ii,jj,kk)= cv_approx;
                    testStruct.dudp(ii,jj,kk)= dudp;
                    testStruct.dudp_approx(ii,jj,kk)= dudp_approx;
                    testStruct.dudphi(ii,jj,kk)= dudphi;
                    testStruct.dudphi_approx(ii,jj,kk)= dudphi_approx;
                    testStruct.dudxf(ii,jj,kk)= dudxf;
                    testStruct.dudxa(ii,jj,kk)= dudxa;
                    testStruct.dudxfa_approx(ii,jj,kk)= dudxfa_approx;
                    if 0
                        figure;
                        plot(reshape(testStruct.cv_approx,1,[]),...
                             reshape(testStruct.cv,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dudp_approx,1,[]),...
                             reshape(testStruct.dudp,1,[]),'.')
                        figure;
                        plot(reshape(testStruct.dudphi_approx,1,[]),...
                             reshape(testStruct.dudphi,1,[]),'.')                    
                        figure;
                        plot(reshape(testStruct.dudxfa_approx,1,[]),...
                             reshape(testStruct.dudxf-testStruct.dudxa,1,[]),'.')                        
                    end
                    
                    Teststruct.T(ii,jj,kk) = T;
                    testStruct.p(ii,jj,kk) = p;
                    testStruct.phi(ii,jj,kk) = phi;
                end
            end
        end
    end
    close(w8handle)

    %chemPropU.T   = (250:12.5:2000);
    %chemPropU.T   = (250:25:2000);
    %chemPropU.T   = (250:5:2000);
    chemPropU.T   = (250:50:2000);
    %setPhi(1e-7)
    [X_a] = chemEqSolve(500,1e5);
    %X_a = X_a.*(X_a>1e-5);
    %X_a = X_a/sum(X_a);
    X_a = zeros(size(X_a));
    X_a(find(strcmp(SELECTED_NAMES,'O2'))) = 1;
    X_a(find(strcmp(SELECTED_NAMES,'N2'))) = 3.773;
    X_a = X_a/sum(X_a);
    for ii = 1:length(chemPropU.T)
        T = chemPropU.T(ii);
        [cp_f,h_f,dummy,M_f] = thermPropFuel(T);
        cp_f       = cp_f/M_f;
        h_f        = h_f/M_f;
        [cp_tilde_all_a,h_tilde_all_a] = thermPropAll(T);
        M_a        = MOLEC_WEIGHT*X_a;
        h_a        = X_a'*h_tilde_all_a/M_a;
        cp_tilde_a = X_a'*cp_tilde_all_a;
        cp_a       = cp_tilde_a/M_a;

        chemPropU.Mf         = M_f;
        chemPropU.Ma         = M_a;
        chemPropU.cpf(ii)    = cp_f;
        chemPropU.hf(ii)     = h_f;
        chemPropU.cpa(ii)    = cp_a;
        chemPropU.ha(ii)     = h_a; 
    end
    chemPropU = orderfields(chemPropU,fnamesU);    
    chemPropB = orderfields(chemPropB,fnamesB); 
else
    error('The requested fuel ''%s'' is unavalable',theFuel);
end

psFuelData.AFs = AFs;

T= gammaApprox_temp;
p= gammaApprox_p;
setPhi(1)
[X,dXdT,dXdp,dXdb] = chemEqSolve(T,gammaApprox_p);
[cp_tilde_all,h_tilde_all] = thermPropAll(T);
M        = MOLEC_WEIGHT*X;
R        = R_tilde/M;               
dMdT     = MOLEC_WEIGHT*dXdT;
cp_tilde = X'*cp_tilde_all+dXdT'*h_tilde_all;
cp       = (cp_tilde-(dMdT*X'*h_tilde_all)/M)/M;
cv       = cp-R+R*T/M*dMdT;
psFuelData.gammaApprox = cp/cv;
psFuelData.gammaApprox_T=gammaApprox_temp;
psFuelData.gammaApprox_p=gammaApprox_p;
psFuelData.T_frozen = T_frozen;
psFuelData.p_frozen = p_frozen;
psFuelData.chemPropB = chemPropB;
psFuelData.chemPropU = chemPropU;
psFuelData.FuelName = theFuel;
%psFuelData.FuelName = 'Isooctane_T100_K_grid'
%psFuelData.FuelName = 'Isooctane_T12_5_K_grid'
save psFuelData psFuelData

eval(['!mv psFuelData.mat ' theFuel ])
