%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start psPack if not already started
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psStartUp('startUp',0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load simulation, this overrides the gui 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psLoadSimStruct=load('psBombSimulation');
testedFuels = {'Isooctane','Diesel'};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cycle through fuels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for ii = 1:length(testedFuels)
    clear psCalcMixXgc
    clear psThermProp
    clear psCalcPhi
    clear psCalcReactXgc
    clear psCalcXgc

    theFuel=load(testedFuels{ii});
    psLoadSimStruct.psSimStruct.psThermProp = theFuel.psThermPropData;
    psLoadSimStruct.psSimStruct.psThermProp.varInfo.initialized=1;
    psStartUp('loadSimStruct',psLoadSimStruct.psSimStruct);
    psStartUp('updateControlStruct');
    psStartUp('updateMainWindow');
    psSimStruct.psGeometry.V = 2e-3;
    psSimStruct.psHeat.h = 1e4;
    psSimStruct.psModel.phiMix_u = 0.1;
    psSimStruct.psVibe.eta = 1-1e-5;
    psSimStruct.psModel = psCalcDepVars(psSimStruct.psModel);


    warning off
    sim('psModel');
    warning on
    NaNIdx = find(isnan(psSimOut.dQ),1,'first');

    if ii == 1
        clear h_b cp_b cv_b R_b dRdp_b dRdT_b dudp_b dudX_b dRdX
        clear h_u cp_u cv_u R_u dRdp_u dRdT_u dudp_u dudX_u dRdX
        for kk = 1:(NaNIdx)
            [h_b(kk) cp_b(kk) cv_b(kk) R_b(kk) dRdp_b(kk) dRdT_b(kk) dudp_b(kk) dudX_b(kk,:) dRdX(kk,:)]=psThermProp(psSimOut.pcb(kk),psSimOut.Tb(kk),psCalcReactXgc(psSimStruct.psModel.Xc0),ones(1,9));
            [h_u(kk) cp_u(kk) cv_u(kk) R_u(kk) dRdp_u(kk) dRdT_u(kk) dudp_u(kk) dudX_u(kk,:) dRdX(kk,:)]=psThermProp(psSimOut.pcb(kk),psSimOut.Tu(kk),psSimStruct.psModel.Xc0,ones(1,9));        
            xB(kk) = psVibe(psSimOut.theta(kk));
        end
        Tb = psSimOut.Tb(1:NaNIdx)';
        Tu = psSimOut.Tu(1:NaNIdx)';
        Tav = Tu.*(1-xB)+Tb.*(xB);
        u = (h_b-R_b.*Tb).*xB+(h_u-R_u.*Tu).*(1-xB);
        h = h_b.*xB+h_u.*(1-xB);
        disp('--------------------')
        disp('Note that there are subtile differences between the data ')
        disp('from these simulations and the data that is needed to ')
        disp('calculate the qhv numbers in Table 2.3 of Per Obergs thesis')
        disp('...Pausing...')
        pause
        save simResults.mat u Tav h h_b R_b Tb xB h_u R_u Tu 
    end

    
    dQTot = trapz(psSimOut.time(1:(NaNIdx-1)),psSimOut.dQ(1:(NaNIdx-1)));

    p0  = psSimStruct.psModel.p0;
    V0  = psSimStruct.psGeometry.V;
    T0  = psSimStruct.psModel.T0;
    Xc0 = psSimStruct.psModel.Xc0;
    [h cp cv R dRdp dRdT dudp dudX dRdX]=psThermProp(p0,T0,Xc0,ones(1,9));

    %xf =Xc0(5) 
    xf =Xc0(1); 
    mf=xf*p0*V0/R/T0;

    qHV(ii) = dQTot/mf
end
