function psBomb(block)
setup(block);
%endfunction

%% Function: setup ===================================================
%% Abstract:
%%   Set up the S-function block's basic characteristics such as:
%%   - Input ports
%%   - Output ports
%%   - Dialog parameters
%%   - Options
%% 
%%   Required         : Yes
%%   C-Mex counterpart: mdlInitializeSizes
%%
function setup(block)
  %disp(sprintf('Running ps2ZoneCylinder:Setup at time: %d',NaN))
  % Register number of ports

  global psSimStruct
  K = psSimStruct.psThermProp.K;
      
  block.NumInputPorts  = 1;
  block.NumOutputPorts = 1;

  
  % In signals: 
  % 1) theta
  % Utsignaler:
  % 1) DiagOut (Cont states Disc states and dQ)

  % Cont States are:
  %  X = [p,Vb,Tb,Vu,Tu,-Xgc_u-,-Xgc_b-]
  % Disc States are:
  %   
  
  
  % Update sker efter output i varje steg. Dvs saker händer i följande ordning:
  % 1) Setup
  % 2) SetInPortFrameData
  % 3) DoPostPropSetup
  % 4) Start (time exists from hereon)
  % 5) Init 
  % 6) Output
  % 7) Update
  % 8) Repeat 6-7 and possibly 5-7
  % Start kan alltså anvädas första ggn modellen startas för att
  % avgöra vars vi är  medans initialize kan köras varje gång vi kommer
  % tillbaks från ett disablat tillstånd.

  % None of this dynamic inport stuff
  block.SetPreCompPortInfoToDefaults;
     
  %%%%%%%%%%%%%%%%
  % Input ports
  %%%%%%%%%%%%%%%%
  % Data type
  block.InputPort(1).DatatypeID  = 0;  % double

  % No complex signals
  block.InputPort(1).Complexity  = 'Real';

  % Dimensions are fixed
  block.InputPort(1).Dimensions  = 1;

  % No direct feedthrough
  block.InputPort(1).DirectFeedthrough = false;
  
  % Never accept frame data... 
  block.InputPort(1).SamplingMode  = 0;

  %%%%%%%%%%%%%%%%
  % Output ports
  %%%%%%%%%%%%%%%%  
  % Data type
  block.OutputPort(1).DatatypeID  = 0; % double

  % No complex signals
  block.OutputPort(1).Complexity  = 'Real';
  
  % Dimensions are fixed
  block.OutputPort(1).Dimensions  = 5+K*2+1;

  % Never accept frame data... 
  block.OutputPort(1).SamplingMode  = 0;
  
  % Register parameters
  block.NumDialogPrms     = 0;
  %  block.DialogPrmsTunable = {'Tunable','Nontunable','SimOnlyTunable'};

  % Register sample times
  %  [0 offset]            : Continuous sample time
  %  [positive_num offset] : Discrete sample time
  %
  %  [-1, 0]               : Port-based sample time
  %  [-2, 0]               : Variable sample time
  block.SampleTimes = [0 0];
  
  % Cont States are:
  %  X = [p,Vb,Tb,Vu,Tu,-Xc_u-,-Xc_b-]
  % Disc States are:
  %   pivc,Vivc,Tivc
  block.NumContStates = 5+2*K;
  
  %% -----------------------------------------------------------------
  %% Options
  %% -----------------------------------------------------------------
  % Specify if Accelerator should use TLC or call back into 
  % M-file
  block.SetAccelRunOnTLC(false);
  
  %% -----------------------------------------------------------------
  %% Register methods called during update diagram/compilation
  %% -----------------------------------------------------------------
  
  %% 
  %% CheckParameters:
  %%   Functionality    : Called in order to allow validation of
  %%                      block's dialog parameters. User is 
  %%                      responsible for calling this method
  %%                      explicitly at the start of the setup method
  %%   C-Mex counterpart: mdlCheckParameters
  %%
  block.RegBlockMethod('CheckParameters', @CheckPrms);

  %%
  %% PostPropagationSetup:
  %%   Functionality    : Setup work areas and state variables. Can
  %%                      also register run-time methods here
  %%   C-Mex counterpart: mdlSetWorkWidths
  %%
  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);

  %% -----------------------------------------------------------------
  %% Register methods called at run-time
  %% -----------------------------------------------------------------
  
  %% 
  %% ProcessParameters:
  %%   Functionality    : Called in order to allow update of run-time
  %%                      parameters
  %%   C-Mex counterpart: mdlProcessParameters
  %%  
  block.RegBlockMethod('ProcessParameters', @ProcessPrms);

  %% 
  %% InitializeConditions:
  %%   Functionality    : Called in order to initalize state and work
  %%                      area values
  %%   C-Mex counterpart: mdlInitializeConditions
  %% 
  block.RegBlockMethod('InitializeConditions', @InitializeConditions);
  
  %% 
  %% Start:
  %%   Functionality    : Called in order to initalize state and work
  %%                      area values
  %%   C-Mex counterpart: mdlStart
  %%
  block.RegBlockMethod('Start', @Start);

  %% 
  %% Outputs:
  %%   Functionality    : Called to generate block outputs in
  %%                      simulation step
  %%   C-Mex counterpart: mdlOutputs
  %%
  block.RegBlockMethod('Outputs', @Outputs);

  %% 
  %% Update:
  %%   Functionality    : Called to update discrete states
  %%                      during simulation step
  %%   C-Mex counterpart: mdlUpdate
  %%
  block.RegBlockMethod('Update', @Update);

  %% 
  %% Derivatives:
  %%   Functionality    : Called to update derivatives of
  %%                      continuous states during simulation step
  %%   C-Mex counterpart: mdlDerivatives
  %%
  block.RegBlockMethod('Derivatives', @Derivatives);
  

  %% 
  %% ZeroCrossings:
  %%   Functionality    : For S-functions with continuous sample
  %%                      time, if input signals have discontinuities,
  %%                      this method is called to detect 
  %%                      discontinuities
  %%   C-Mex counterpart: mdlZeroCrossings
  %%
  block.RegBlockMethod('ZeroCrossings', @ZeroCrosssings);
  
  %% 
  %% SimStatusChange:
  %%   Functionality    : Called when simulation goes to pause mode
  %%                      or continnues from pause mode
  %%   C-Mex counterpart: mdlSimStatusChange
  %%
  block.RegBlockMethod('SimStatusChange', @SimStatusChange);
  
  %% 
  %% Terminate:
  %%   Functionality    : Called at the end of simulation for cleanup
  %%   C-Mex counterpart: mdlTerminate
  %%
  block.RegBlockMethod('Terminate', @Terminate);

  %% -----------------------------------------------------------------
  %% Register methods called during code generation
  %% -----------------------------------------------------------------
  
  %%
  %% WriteRTW:
  %%   Functionality    : Write specific information to RTW file
  %%   C-Mex counterpart: mdlRTW
  %%
  block.RegBlockMethod('WriteRTW', @WriteRTW);

%endfunction

%% -------------------------------------------------------------------
%% The local functions below are provided for illustrative purposes
%% to show how you may implement the various block methods listed
%% above.
%% -------------------------------------------------------------------

function CheckPrms(block)
    %disp(sprintf('Running ps2ZoneCylinder:CheckPrms at time: %d',NaN))
%  a = block.DialogPrm(1).Data;
%  if ~strcmp(class(a), 'double')
%    error('Invalid parameter');
%  end
  
%endfunction

function ProcessPrms(block)
  %disp(sprintf('Running ps2ZoneCylinder:ProcessPrms at time: %d',block.currentTime))
  block.AutoUpdateRuntimePrms;
 
%endfunction
  
function DoPostPropSetup(block)
    %% Register all tunable parameters as runtime parameters.
    block.AutoRegRuntimePrms;

%endfunction

function InitializeConditions(block)
  global psSharedSimData
  global psSimStruct
  %CylNbr = block.DialogPrm(2).Data;
  
  %disp(sprintf('Running ps2ZoneCylinder:InitConditions at time: %d',...
  %             block.currentTime));
  currentCa = block.currentTime*psSimStruct.psSimPar.w;
  
  initStates = psInitCombustion(currentCa);
  % X0=[p,Vb,Tb,Vu,Tu,-Xgc_b-,-Xgc_u-];
  block.ContStates.Data = initStates;
%endfunction

function Start(block)

%endfunction

function WriteRTW(block)
   block.WriteRTWParam('matrix', 'M',    [1 2; 3 4]);
   block.WriteRTWParam('string', 'Mode', 'Auto');
   
%endfunction

function Outputs(block)
%  %disp(sprintf('Running ps2ZoneCylinder:Outputs at time: %d',...
%                block.currentTime))
  global psSimStruct
  X     = block.ContStates.Data';
  theta = block.InputPort(1).Data(1);

  if psVibe(theta) >= psSimStruct.psVibe.eta || psSimStruct.psVibe.earlyHeatTransfer;
      A = X([2 4])'/sum(X([2 4]))*4*pi*(3*psSimStruct.psGeometry.V/4/pi)^(2/3);
      dQ = A.*psSimStruct.psHeat.h.*(X([3 5])'-psSimStruct.psModel.T_Wall);
  else
      dQ = zeros(2,1);
  end

  % Diag port
  block.OutputPort(1).Data = ...
      [block.ContStates.Data ;...
       sum(dQ)]; 

%endfunction

function Update(block)
%endfunction

function Derivatives(block)
%   disp(sprintf('Running ps2ZoneCylinder:Derivatives at time: %d',...
%                 block.currentTime))

  theta    = block.InputPort(1).Data(1);

  global psSimStruct
  K = psSimStruct.psThermProp.K;
  % Get some parameters
  w = psSimStruct.psSimPar.w;

 
  % Pack the states into X
  %  X = [p,Vb,Tb,Vu,Tu,-Xc_u-,-Xc_b-]';
  X     = block.ContStates.Data';
  p     = X(1);
  Vb    = X(2);
  Tb    = X(3);
  Vu    = X(4);
  Tu    = X(5);
  XpVT  = X(1:5)';
  Xgc_b = block.ContStates.Data(5+(1:K))';
  Xgc_u = block.ContStates.Data(5+K+(1:K))';

  Xgc = [Xgc_b ; Xgc_u];

  % Calculate thermochemical properties: h, cp, R, dRdp, dRdT
  % All zones and all parts have the same phi so just set them rigt
  % here. 
  [dummy1 dumy2 dummy3 R] = psThermProp(p,[Tb; Tu],Xgc,[0 0 0 1 0 0 0 0 0]);   
  % Define massflow to and from the zones   
  % dm(i,j) is the massflow from zone j to zone i.
  % Massflows entering zone i have a positive sign.

%Note:
% Anledningen till att man vill ha två zoner under förbränningsfasen är
% att man vill kunna beräkna NOx
  m_b = p*Vb/R(1)/Tb;
  m_u = p*Vu/R(2)/Tu;
  m_tot = m_b+m_u;
%  if ~isfinite(m_tot)
%    out = realmax*ones(2*zones+2,1);
%    return
%  end
  
  dm = zeros(2,2);
  dm(1,2) = m_tot*w*psDVibe(theta);
  dm(2,1) = -dm(1,2);

  if psSimStruct.psVibe.eta || psSimStruct.psVibe.earlyHeatTransfer
      A = X([2 4])'/sum(X([2 4]))*4*pi*(3*psSimStruct.psGeometry.V/4/pi)^(2/3);
      dQ = A.*psSimStruct.psHeat.h.*(X([3 5])'-psSimStruct.psModel.T_Wall);
  else
      dQ = zeros(2,1);
  end
  % Define total derivative of total cylinder Volume
  dV = w*psDVolume(theta);
  
  Xgc_to = psCalcReactXgc(Xgc);

  [dXpVT dXgc]=  psDX(XpVT,Xgc,Xgc_to,dV,dm,dQ,zeros(2,2),zeros(2,K,2),zeros(2,K,2),zeros(2,2));  
  if ~isreal(sum(dXpVT)) || ~isreal(sum(sum(dXgc))) || isnan(sum(dXpVT)) || isnan(sum(sum(dXgc)))
      % This means that the there is something wrong. Most likely it's
      % the tempearature that is negative, or the motored pressure that
      % is larger than the real pressure. This occurs sometimes for very
      % small zones when simulink tries to take a very large
      % step. Therefore  The values could be set to NaN so that Simulink
      % detects the error and takes a shorter step
      dXpVT = ones(size(X(1:5)'))*1e10;
      dXgc  = ones(size(dXgc))*1e10;
  end

  
  block.Derivatives.Data = [dXpVT(:) ;dXgc(1,:)';dXgc(2,:)'];

%endfunction

function ZeroCrosssings(block)
  % Eventuellt lägga till lite kollar för zero-cross hjälp.
  % Tillstånden är:
  % - p,Vbg,Tbg,Vfm,Tfm,Tav,phi_bg,phiRes_bg,xRes_bg,phi_fm,phiRes_fm,xRes_fm
  % och det som kan hända med dem är att Volymerna kan närma sig gränsen
  % för vad som är acceptabel storlek. De viktigaste eventsen är redan
  % omhändertagna men även events som ändring av riktning på flöden mm
  % är av intresse. Ett bra sätt är att kolla efter if-satser i
  % derivata-uträkningen och sätta zero-cross-detection på signalerna i
  % omgivningen som har med signalen att göra, om det är viktigt vill säga.
%endfunction

function SimStatusChange(block, s)
  
  if s == 0
    disp('Pause has been called');
  elseif s == 1
    disp('Continue has been called');
  end
  
%endfunction
    
function Terminate(block)
%endfunction
 
