function psToWorkspace(block)
setup(block);
%endfunction

%% Function: setup ===================================================
%% Abstract:
%%   Set up the S-function block's basic characteristics such as:
%%   - Input ports
%%   - Output ports
%%   - Dialog parameters
%%   - Options
%% 
%%   Required         : Yes
%%   C-Mex counterpart: mdlInitializeSizes
%%
function setup(block)
  %disp(sprintf('Running psToWorkspace:Setup at time: %d',NaN))
  % Register number of ports
  block.NumInputPorts  = 2;
  block.NumOutputPorts = 0;

  % Update sker efter output i varje steg. Dvs saker händer i följande ordning:
  % 1) Setup
  % 2) SetInPortFrameData
  % 3) DoPostPropSetup
  % 4) Start (time exists from hereon)
  % 5) Init 
  % 6) Output
  % 7) Update
  % 8) Repeat 6-7 and possibly 5-7
  % Start kan alltså anvädas första ggn modellen startas för att
  % avgöra vars vi är  medans initialize kan köras varje gång vi kommer
  % tillbaks från ett disablat tillstånd.

  % None of this dynamic inport stuff
  block.SetPreCompPortInfoToDefaults;
     

  % Register parameters
  block.NumDialogPrms     = 3;
  %  block.DialogPrmsTunable = {'Tunable','Nontunable','SimOnlyTunable'};

  %%%%%%%%%%%%%%%%
  % Input ports
  %%%%%%%%%%%%%%%%
  % Port 1
  % Data type
  block.InputPort(1).DatatypeID  = 0;  % double
  % No complex signals
  block.InputPort(1).Complexity  = 'Real';
  % Dimensions are fixed
  block.InputPort(1).Dimensions  = 1;
  % No direct feedthrough
  block.InputPort(1).DirectFeedthrough = true;  
  % Never accept frame data... 
  block.InputPort(1).SamplingMode  = 0;

  
  % Port 2
  % Data type
  block.InputPort(2).DatatypeID  = 0;  % double
  % No complex signals
  block.InputPort(2).Complexity  = 'Real';
  % Dimensions are fixed
  block.InputPort(2).Dimensions  = block.DialogPrm(1).Data;
  % Direct feedthrough
  block.InputPort(2).DirectFeedthrough = true;
  % Never accept frame data... 
  block.InputPort(2).SamplingMode  = 0;


  % Register sample times
  %  [0 offset]            : Continuous sample time
  %  [positive_num offset] : Discrete sample time
  %
  %  [-1, 0]               : Port-based sample time
  %  [-2, 0]               : Variable sample time
  block.SampleTimes = [0 1]; 
  
  % No cont states
  block.NumContStates = 0;
  
  %% -----------------------------------------------------------------
  %% Options
  %% -----------------------------------------------------------------
  % Specify if Accelerator should use TLC or call back into 
  % M-file
  block.SetAccelRunOnTLC(false);
  
  %% -----------------------------------------------------------------
  %% Register methods called during update diagram/compilation
  %% -----------------------------------------------------------------
  
  %% 
  %% CheckParameters:
  %%   Functionality    : Called in order to allow validation of
  %%                      block's dialog parameters. User is 
  %%                      responsible for calling this method
  %%                      explicitly at the start of the setup method
  %%   C-Mex counterpart: mdlCheckParameters
  %%
  block.RegBlockMethod('CheckParameters', @CheckPrms);

  %%
  %% PostPropagationSetup:
  %%   Functionality    : Setup work areas and state variables. Can
  %%                      also register run-time methods here
  %%   C-Mex counterpart: mdlSetWorkWidths
  %%
  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);

  %% -----------------------------------------------------------------
  %% Register methods called at run-time
  %% -----------------------------------------------------------------
  
  %% 
  %% ProcessParameters:
  %%   Functionality    : Called in order to allow update of run-time
  %%                      parameters
  %%   C-Mex counterpart: mdlProcessParameters
  %%  
  block.RegBlockMethod('ProcessParameters', @ProcessPrms);

  %% 
  %% InitializeConditions:
  %%   Functionality    : Called in order to initalize state and work
  %%                      area values
  %%   C-Mex counterpart: mdlInitializeConditions
  %% 
  block.RegBlockMethod('InitializeConditions', @InitializeConditions);
  
  %% 
  %% Start:
  %%   Functionality    : Called in order to initalize state and work
  %%                      area values
  %%   C-Mex counterpart: mdlStart
  %%
  block.RegBlockMethod('Start', @Start);

  %% 
  %% Outputs:
  %%   Functionality    : Called to generate block outputs in
  %%                      simulation step
  %%   C-Mex counterpart: mdlOutputs
  %%
  block.RegBlockMethod('Outputs', @Outputs);

  %% 
  %% Update:
  %%   Functionality    : Called to update discrete states
  %%                      during simulation step
  %%   C-Mex counterpart: mdlUpdate
  %%
  block.RegBlockMethod('Update', @Update);

  %% 
  %% Derivatives:
  %%   Functionality    : Called to update derivatives of
  %%                      continuous states during simulation step
  %%   C-Mex counterpart: mdlDerivatives
  %%
  block.RegBlockMethod('Derivatives', @Derivatives);
  

  %% 
  %% ZeroCrossings:
  %%   Functionality    : For S-functions with continuous sample
  %%                      time, if input signals have discontinuities,
  %%                      this method is called to detect 
  %%                      discontinuities
  %%   C-Mex counterpart: mdlZeroCrossings
  %%
  block.RegBlockMethod('ZeroCrossings', @ZeroCrosssings);
  
  %% 
  %% SimStatusChange:
  %%   Functionality    : Called when simulation goes to pause mode
  %%                      or continnues from pause mode
  %%   C-Mex counterpart: mdlSimStatusChange
  %%
  block.RegBlockMethod('SimStatusChange', @SimStatusChange);
  
  %% 
  %% Terminate:
  %%   Functionality    : Called at the end of simulation for cleanup
  %%   C-Mex counterpart: mdlTerminate
  %%
  block.RegBlockMethod('Terminate', @Terminate);

  %% -----------------------------------------------------------------
  %% Register methods called during code generation
  %% -----------------------------------------------------------------
  
  %%
  %% WriteRTW:
  %%   Functionality    : Write specific information to RTW file
  %%   C-Mex counterpart: mdlRTW
  %%
  block.RegBlockMethod('WriteRTW', @WriteRTW);

%endfunction

%% -------------------------------------------------------------------
%% The local functions below are provided for illustrative purposes
%% to show how you may implement the various block methods listed
%% above.
%% -------------------------------------------------------------------

function CheckPrms(block)
    %disp(sprintf('Running psToWorkspace:CheckPrms at time: %d',NaN))
%  a = block.DialogPrm(1).Data;
%  if ~strcmp(class(a), 'double')
%    error('Invalid parameter');
%  end
  
%endfunction

function ProcessPrms(block)
  %disp(sprintf('Running psToWorkspace:ProcessPrms at time: %d',block.currentTime))
  block.AutoUpdateRuntimePrms;
 
%endfunction
  
function DoPostPropSetup(block)
  %disp(sprintf('Running psToWorkspace:DoPostPropSetup at time: %d',NaN))

  % Disc States:
  %   1-2) flowFlagUp,flowFlagDown (För hyseteres när typen av flöde ändras)
  %   3) phi_bg,phiRes_bg,xRes_bg,phi_fm,phiRes_fm,xRes_fm 
  %   4) mdlStartTime

  
  block.NumDworks = 1;
  block.Dwork(1).Name            = 'WasActive';
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 0;      % double
  block.Dwork(1).Complexity      = 'Real'; % real
  block.Dwork(1).UsedAsDiscState = true;

  
  %% Register all tunable parameters as runtime parameters.
  block.AutoRegRuntimePrms;

%endfunction

function InitializeConditions(block)
  % WasActive flag is set to zero
      block.Dwork(1).Data = 0;
%endfunction

function Start(block)
  global psSimOut
 
  % Define chunkSize
  chunkSize =100;

  %  SigNbr
  SigNbr = block.DialogPrm(3).Data;
  
  % Get SignalNames
  param2 = block.DialogPrm(2).Data;
  startPoints = [1 find(param2 == 32)+1];
  for ii = 1:length(startPoints)
      if ii == length(startPoints)
          SignalNames{ii}=char(param2(startPoints(ii):end));
      else
          SignalNames{ii}=char(param2(startPoints(ii):(startPoints(ii+1)-2)));
      end
  end
  
  % Create new if it does not exist or if it is already used.
  if isempty(psSimOut) || ~isnan(psSimOut.time(1)) || ~isfield(psSimOut,'simData')
      psSimOut = [];
      psSimOut.simData = [];
      psSimOut.time = NaN*ones(chunkSize,1);
      psSimOut.simData.timeIdx = 0;
  end
  % Just add the fields that arent there already, don't care if
  % something is overwritten since nothing should be able to be there
  for ii = 1:length(SignalNames)
      if ~strcmp(SignalNames{ii},'-')
          psSimOut.(SignalNames{ii})(:,SigNbr) = psSimOut.time;
      end
  end

  psSimOut = orderfields(psSimOut);
%endfunction

function WriteRTW(block)
  
   block.WriteRTWParam('matrix', 'M',    [1 2; 3 4]);
   block.WriteRTWParam('string', 'Mode', 'Auto');
   
%endfunction

function Outputs(block)
  global psSimOut

  % Define chunkSize
  chunkSize = 100;

  %  SigNbr
  SigNbr = block.DialogPrm(3).Data;
  
  
  % If all space is used up, reserve some more
  if psSimOut.simData.timeIdx == length(psSimOut.time)
    theNames = fieldnames(psSimOut);
    for ii = 1:length(theNames)
        if ~strcmp(theNames{ii},'simData')
       psSimOut.(theNames{ii}) = [psSimOut.(theNames{ii}); NaN*ones(chunkSize,size(psSimOut.(theNames{ii}),2))];
        end
    end
    theTime = clock;
    disp(sprintf('%i:%i:%i Chunk full: Reserving more space',theTime(4),theTime(5),round(theTime(6))));
    
  end

  % Get SignalNames
  param2 = block.DialogPrm(2).Data;
  startPoints = [1 find(param2 == 32)+1];
  for ii = 1:length(startPoints)
      if ii == length(startPoints)
          SignalNames{ii}=char(param2(startPoints(ii):end));
      else
          SignalNames{ii}=char(param2(startPoints(ii):(startPoints(ii+1)-2)));
      end
  end
  if psSimOut.simData.timeIdx && psSimOut.time(psSimOut.simData.timeIdx) == block.CurrentTime
    %%%%%%%%%%%%%%%
    % OLD TIME
    %%%%%%%%%%%%%%%
    % Some things are already stored and we just need to fillup with
    % some more stuff. If a signal is specified more than once a warning
    % is given.
    for ii = 1:length(SignalNames)
      if ~strcmp(SignalNames{ii},'-')
        if ~isnan(psSimOut.(SignalNames{ii})(psSimOut.simData.timeIdx,SigNbr))
          warning('psToWorkspave:Outputs',...
                  sprintf(['You have specified the same output signal'...
                           ' in more than one block. Latest is: %i '],SigNbr));
        end
        if block.InputPort(1).Data || block.Dwork(1).Data
          psSimOut.(SignalNames{ii})(psSimOut.simData.timeIdx,SigNbr) = ...
              block.InputPort(2).Data(ii);
        else
            psSimOut.(SignalNames{ii})(psSimOut.simData.timeIdx,SigNbr) = NaN;
        end
      end
    end
  else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % New time
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    psSimOut.simData.timeIdx =  psSimOut.simData.timeIdx+1;
    psSimOut.time(psSimOut.simData.timeIdx) = block.currentTime;
    for ii = 1:length(SignalNames)
      if ~strcmp(SignalNames{ii},'-')
        if block.InputPort(1).Data  || block.Dwork(1).Data
            psSimOut.(SignalNames{ii})(psSimOut.simData.timeIdx,SigNbr) = ...
                block.InputPort(2).Data(ii);
        else 
            psSimOut.(SignalNames{ii})(psSimOut.simData.timeIdx,SigNbr) = NaN;
        end
      end
    end
  end
%endfunction

function Update(block)
  block.Dwork(1).Data = block.InputPort(1).Data;
%endfunction

function Derivatives(block)
%endfunction

function ZeroCrosssings(block)
  % Eventuellt lägga till lite kollar för zero-cross hjälp.
  % Tillstånden är:
  % - p,Vbg,Tbg,Vfm,Tfm,Tav,phi_bg,phiRes_bg,xRes_bg,phi_fm,phiRes_fm,xRes_fm
  % och det som kan hända med dem är att Volymerna kan närma sig gränsen
  % för vad som är acceptabel storlek. De viktigaste eventsen är redan
  % omhändertagna men även events som ändring av riktning på flöden mm
  % är av intresse. Ett bra sätt är att kolla efter if-satser i
  % derivata-uträkningen och sätta zero-cross-detection på signalerna i
  % omgivningen som har med signalen att göra, om det är viktigt vill säga.
%endfunction

function SimStatusChange(block, s)
  
  if s == 0
    disp('Pause has been called');
  elseif s == 1
    disp('Continue has been called');
  end
  
%endfunction
    
function Terminate(block)
    global psSimOut
    if isfield(psSimOut,'simData')
        psSimOut = rmfield(psSimOut,'simData');
    end
%endfunction
 
